! Atom relaxation algorithm by Conjugate Gradient method & Line minimization
!
!  Numerical recipes : Sec.10.5-6
!
!  Last update 21/02/2007

      subroutine atom_relax_cg
      use variables
      use constants
      implicit double precision (a-h,o-z)
      double precision :: gamcg,gamcg1,gamcg2
      integer i

!     1step : initial position
!     2step : trial step
!     3step : corrector step
!     4step : trial step
!     5step : corrector step
!     ...

      if (step==1) then
         allocate (sgx(n,2),sgy(n,2),sgz(n,2))
         allocate (shx(n,2),shy(n,2),shz(n,2))
         allocate (rstx(n),rsty(n),rstz(n))
         lamtr=lam_init
         goto 999
      endif

!-----Data
      ! Initial step (PREVIOUS step data)
      if (step==2) then
         cgpot0=enpot
         cggrad0=0.0d0
         print*,cgpot0,cggrad0
         do i=1,n
            cggrad0=cggrad0-(fx(i)*fx(i)+fy(i)*fy(i)+fz(i)*fz(i))
         enddo
         normsh=dsqrt(dabs(cggrad0))
         cggrad0=cggrad0/normsh
         do i=1,n
            rstx(i)=rx(i)
            rsty(i)=ry(i)
            rstz(i)=rz(i)
         enddo
      ! Trial step (PREVIOUS step data)
      elseif (mod(step,2)==1) then 
         cgpot1=enpot
         cggrad1=0.0d0
         do i=1,n
            cggrad1=cggrad1-(fx(i)*shx(i,1)+fy(i)*shy(i,1)+fz(i)*shz(i,1))
         enddo
         cggrad1=cggrad1/normsh
      ! Corrector step (PREVIOUS step data)
      elseif (mod(step,2)==0) then
         cgpot0=enpot
         cggrad0=0.0d0
         do i=1,n
            cggrad0=cggrad0-(fx(i)*shx(i,1)+fy(i)*shy(i,1)+fz(i)*shz(i,1))
         enddo
         cggrad0=cggrad0/normsh
         do i=1,n
            rstx(i)=rx(i)
            rsty(i)=ry(i)
            rstz(i)=rz(i)
         enddo
      endif


!-----Search vector shxyz
      if (mod(step,2)==0) then  ! trial step
         do i=1,n
            sgx(i,1)=fx(i)
            sgy(i,1)=fy(i)
            sgz(i,1)=fz(i)
            if (mfx(i)=='F') then;sgx(i,1)=0.0d0;endif
            if (mfy(i)=='F') then;sgy(i,1)=0.0d0;endif
            if (mfz(i)=='F') then;sgz(i,1)=0.0d0;endif
         enddo
         ! set GAMMA
         if (step==2) then
            gamcg=0.0d0
         else
            gamcg1=0.0d0
            gamcg2=0.0d0
            do i=1,n
               gamcg1=gamcg1+(sgx(i,1)-sgx(i,2))*sgx(i,1) &
                    +(sgy(i,1)-sgy(i,2))*sgy(i,1)+(sgz(i,1)-sgz(i,2))*sgz(i,1)
               gamcg2=gamcg2+sgx(i,2)*sgx(i,2)+sgy(i,2)*sgy(i,2)+sgz(i,2)*sgz(i,2)
            enddo
            gamcg=gamcg1/gamcg2
! Steepest Descent method : set gamcg=0
!           gamcg=0.0d0
         endif
         ! set search vector SH
         do i=1,n
            shx(i,1)=sgx(i,1)+gamcg*shx(i,2)
            shy(i,1)=sgy(i,1)+gamcg*shy(i,2)
            shz(i,1)=sgz(i,1)+gamcg*shz(i,2)
         enddo
         normsh=0.0d0
         do i=1,n
            normsh=normsh+shx(i,1)**2+shy(i,1)**2+shz(i,1)**2
         enddo
         normsh=dsqrt(normsh)
         do i=1,n
            sgx(i,2)=sgx(i,1)
            sgy(i,2)=sgy(i,1)
            sgz(i,2)=sgz(i,1)
            shx(i,2)=shx(i,1)
            shy(i,2)=shy(i,1)
            shz(i,2)=shz(i,1)
         enddo
      endif


!-----Line minimization
      ! Trial step
      if (mod(step,2)==0) then 
         do i=1,n
            rx(i)=rstx(i)+shx(i,1)*lamtr
            ry(i)=rsty(i)+shy(i,1)*lamtr
            rz(i)=rstz(i)+shz(i,1)*lamtr
         enddo
      ! Corrector step
      elseif (mod(step,2)==1) then
         call intpl_cg (0.0d0,cgpot0,cggrad0,lamtr*normsh,cgpot1,cggrad1,lamcrr)
         lamcrr=lamcrr/normsh
         lamtr=lamcrr
         ! RESET LAMDA_TRIAL
         if (lamtr <= 1.0d-8) lamtr = lam_init
         do i=1,n
            rx(i)=rstx(i)+shx(i,1)*lamcrr
            ry(i)=rsty(i)+shy(i,1)*lamcrr
            rz(i)=rstz(i)+shz(i,1)*lamcrr
         enddo
      endif
      
999   continue
      end


!-----------------------------------------------------------------------
      subroutine intpl_cg (x1,y1,dy1,x2,y2,dy2,x3)
      real(8) :: x1,y1,dy1,x2,y2,dy2,x3
      real(8) :: a,b,c,d,xx,root
      
!     ** Only 'x1=0.0d0' is acceptable **

!     Cubic funciton interpolation (Default)
      a=-(2.0d0*y2-dy2*x2-dy1*x2-2.0d0*y1)/x2**3
      b=(3.0d0*y2-3.0d0*y1-2.0d0*dy1*x2-dy2*x2)/x2**2
      c=dy1
      d=y1
      root=b**2-3.0d0*a*c
      if (root.gt.0.0d0) then
         xx=(-b+dsqrt(root))/(3.0d0*a)
         ddy=6.0d0*a*xx+2.0d0*b
         if (ddy.gt.0.0d0) then
            x3=xx
         else
            x3=(-b-dsqrt(root))/(3.0d0*a)
         endif
!     Quadratic function (if cubic interpolation failed)
      else
         a=(y2-y1-dy1*x2)/x2**2
         b=dy1
         x3=-b/(2.0d0*a)
         if (a.le.0.0d0) then
            a=-(y2-y1-dy2*x2)/x2**2
            b=(2.0d0*y2-2.0d0*y1-dy2*x2)/x2
            x3=-b/(2.0d0*a)
            if (a.le.0.0d0) x3=x2
         endif
      endif
      end
!-----------------------------------------------------------------------
