! Atom relaxation algorithm by Fast Inertial Relaxation Engine (FIRE)
!
!  E.Bitzek, P.Koshinen, F.Gahler, M.Moseler and P.Gumbsch
!  Physical Review Letters, 97, 170201 (2006) 
!
!  Last update 21/02/2007

      subroutine atom_relax_fire
      use variables
      use constants
      implicit double precision (a-h,o-z)
      double precision :: sp,fnorm,vnorm
      double precision :: ffinc,ffdec,ffalph,fire_alph_ini
      integer i,nfmin

! Parameters for FIRE
      fire_alph_ini=0.10d0
      nfmin=5
      ffinc=1.1d0
      ffdec=0.5d0
      ffalph=0.99d0
      if (step==1) ffdtmax=dt*10.0d0 ! ROUGHLY ESTIMATED VALUE


!-----F1 : P=F*v 
      ifire=ifire+1
      sp=0.0d0
      fnorm=0.0d0
      vnorm=0.0d0
      do i=1,n
         sp = sp + fx(i)*vx(i)+fy(i)*vy(i)+fz(i)*vz(i)
         fnorm = fnorm + fx(i)**2+fy(i)**2+fz(i)**2
         vnorm = vnorm + vx(i)**2+vy(i)**2+vz(i)**2
      enddo
      fnorm=dsqrt(fnorm)
      vnorm=dsqrt(vnorm)

!-----F2 : v = (1-a)*v + a*F*|v|
      do i=1,n
         vx(i)=(1.0d0-fire_alph)*vx(i)+fire_alph*fx(i)/fnorm*vnorm
         vy(i)=(1.0d0-fire_alph)*vy(i)+fire_alph*fy(i)/fnorm*vnorm
         vz(i)=(1.0d0-fire_alph)*vz(i)+fire_alph*fz(i)/fnorm*vnorm
      enddo

!-----F3 : P > 0
      if ((sp > 0.0d0).and.(ifire >= nfmin)) then
         dt = min(dt*ffinc,ffdtmax)
         fire_alph=fire_alph*ffalph
!-----F4 : P <= 0
      elseif (sp <= 0.0d0) then
         ifire = 0
         dt = dt*ffdec
         fire_alph = fire_alph_ini
         vx=0.0d0; vy=0.0d0; vz=0.0d0
      endif

      end
