!----------------------------------------------------------
!
!     Molecular Dynamics Simulation Package for Analysis of Solid Systems
!     (MD-SPASS)
!     Fortran90 Program for Classical Atomistic Simulations
!
!     Developed by Dr. Yoshi Umeno (umeno@iis.u-tokyo.ac.jp)
!     Laboratory of Computational Materials Science and Mechanics
!     Institute of Industrial Science
!     The University of Tokyo
!
!     Partly supported by Takahiro Shimada
!     Materials Science Laboratory
!     Dept. of Mechanical Engineering and Science
!     Graduate School of Engineering, Kyoto University
!
!     Modified from md2005:
!     more potentials (Tersoff, SW)
!     multisystems
!----------------------------------------------------------
      module output
      integer :: logmode = 0
      end module

      module constants
      double precision :: rc = -8.0d-10 ! Cutoff radius [m]
      double precision :: dt = 1.0d-15 ! Time step [s]
      double precision :: a  = 4.04959d-10 ! Lattice constant [m]
      character (2), allocatable :: asp(:) ! Atom species (name)
!      integer, allocatable :: atp(:) ! Atom species (numbering in code)
      integer :: nsp ! number of species
      integer :: multisystem = 0 
      character (15) :: potential_func = 'EAM_MIS' ! Potential type
      character (7) :: potential_arg
      double precision :: am = 1.6726485d-27 ! Mass of proton [kg]
      double precision :: bm = 1.6749543d-27 ! Mass of neutron [kg]
      double precision, allocatable :: wm(:) ! Mass of atom [kg]
      double precision :: wmave ! Averaged mass of atom
      double precision :: pi = 3.141592654d0 ! Pi
      double precision :: bl = 1.380662d-23 ! Boltzmann constant
      double precision :: ev = 1.6021892d-19 ! 1eV=1.6021892d-19 J
      double precision :: elc= 1.6021892d-19  ! elemetary charge e (C)
      double precision :: epsil0=8.854187816d-12 ! Permittivity in vacuum (F/m)
      double precision :: bohr=5.2917706d-11
      double precision :: ang= 1.0d-10 ! 1ang=10^-10 m
      double precision :: gpa= 1.0d9 ! 1GPa=10^9 Pa
      double precision :: mpa= 1.0d6 ! 1MPa=10^6 Pa
      double precision :: relax_factor = 1.0d0
      end module

      module bookkeep_param
      double precision :: frc = 9.0d-10
      integer :: nbk=10
      integer, allocatable :: idpt(:,:),knum(:),irep(:)
      integer :: nbook = 1000000
      integer :: nrpx = 1, nrpy = 1, nrpz = 1 ! # of replica cells to be serched
      integer :: bookkeep_algo = 1 ! Bookkeeping algorithm (1=normal, 2=double)
      end module

      module rp_param
      logical :: rpart = .false.
      integer, allocatable :: ncl(:,:,:),nir(:,:),idpart(:)
      integer :: npcx=8, npcy=3, npcz=3
      integer :: nnor, nnir      
      double precision :: spcx, spcy, spcz
      double precision, allocatable :: coor(:,:)
      end module

      module debug_variables
      logical :: debug = .false.
      double precision, allocatable :: den1(:),den2(:)
      integer, allocatable :: nn(:),nnnum(:,:)
      double precision, allocatable :: distance(:,:),fcom(:,:,:)
      end module

      module variables
      integer :: iwarn = 0      ! Count of warning(s)
      integer :: step = 0       ! Iteration number
      integer :: step_ite =50   ! Iteration to go
      integer :: step_start     ! Start step
      integer :: step_end       ! Last step
      integer :: n              ! Number of atoms
      integer :: vscale = 0     ! Velocity scaling
                                ! 0: no, integer:interval steps to apply
      integer :: damper = 0     ! Atom damping
                                ! 0: no, integer:interval steps to apply
      integer :: naccew = 10    ! Accuracy parameter for Ewald method
      logical :: cellconstx = .false. ! Cell constraint
      logical :: cellconsty = .false. 
      logical :: cellconstz = .false. 

      integer :: algorithm = 8  ! Numerical int. algo.
        ! 0=Velocity Verlet, NVE (Allen & Tildesley)
        ! 1=Gear's P-C, NVE (Allen & Tildesley)
        ! 2=Gear, NVT (Nose thermostat)
        ! 3=Gear, NPH (Andersen pressure control)
        ! 4=Gear, NPT (Nose and Andersen)
        ! 5=Gear, NPH-PR (Parrinello-Rahman)
        ! 6=Gear, NPT-PR (Nose and Parrinello-Rahman)
        ! 7=Stress relaxation by annealing, no atom motion
        ! 8=Stress relaxation by annealing, atom motion by verlet
        ! 107=Gear, Stress relaxation by NPH-PR & breaking
        ! 100=Configuration relaxation by NR
!      double precision, allocatable :: ityp(:) ! Atom type
      double precision, allocatable :: rx(:),ry(:),rz(:) ! Atom positions
      double precision, allocatable :: vx(:),vy(:),vz(:) ! Velocities
      double precision, allocatable :: fx(:),fy(:),fz(:) ! Forces
      double precision, allocatable :: qx(:),qy(:),qz(:) ! Normalized pos
      double precision, allocatable :: ax(:),ay(:),az(:) ! 2nd derivatives
      double precision, allocatable :: bx(:),by(:),bz(:) ! 3rd derivatives
      double precision, allocatable :: cx(:),cy(:),cz(:) ! 4th derivatives
      character (1), allocatable :: mfx(:),mfy(:),mfz(:) ! Atom motion control
      double precision, allocatable :: qpx(:),qpy(:),qpz(:) ! Atom positions for memory
      double precision, allocatable :: tmprx(:,:),tmpry(:,:),tmprz(:,:) ! positions for SM
      double precision, allocatable :: tmpfx(:,:),tmpfy(:,:),tmpfz(:,:) ! forces for SM
      double precision, allocatable :: enpoteach(:)      ! Potential per atom
      double precision, allocatable :: satom(:,:,:) ! Atomic stress
      double precision :: s,sv,sa,sb,sc ! S and its derivatives
      double precision :: sf ! "correct" sa (to calculate corrector)
      double precision :: v,vv,va,vb,vc ! Volume and its deriatives
      double precision :: hmat(3,3),hvmat(3,3),hamat(3,3),hbmat(3,3),hcmat(3,3)
                                ! Lattice matrix and its derivatives
      double precision :: hmat0(3,3) ! Original lattice matrix (for constraint)
      double precision :: hinmat(3,3) ! Inverse Lattice matrix
      double precision :: dmat(3,3) ! for stress calculation
      double precision :: enpot,enkin ! potenrial & kinetic energies
      logical :: pbcx = .true. ,pbcy = .true. ,pbcz = .true.
                                ! Periodic boundary condition
      logical :: continue_md = .false.
      logical :: ini_velocity = .false.

!!! CG
          double precision, allocatable :: rstx(:),rsty(:),rstz(:)
          double precision, allocatable :: sgx(:,:),sgy(:,:),sgz(:,:)
          double precision, allocatable :: shx(:,:),shy(:,:),shz(:,:)
          double precision :: lam_init = 1.0d-3
          double precision :: normsh,lamcrr,lamtr
          double precision :: cgpot0,cggrad0,cgpot1,cggrad1

!!! FIRE
      integer :: ifire = 0
      double precision :: fire_alph = 0.10d0
      double precision :: ffdtmax

! Temperature
      double precision :: temp_set = 200.0d0 ! target temperature
      double precision :: temp_weight = 1.0d0 ! Weight of thermostat
      double precision :: wqr = 1.5d1
      double precision :: wq, temp

! Stress
      double precision :: virx, viry, virz
      double precision :: sgm_set = 0.0d0 ! target stress
      double precision :: sxx=0.0d6, syy=0.0d6, szz=0.0d6 ! target stress
      double precision :: syz=0.0d6, szx=0.0d6, sxy=0.0d6 ! target stress
      double precision :: sgmmat_set(3,3) = 0.0d0 ! target stress
      double precision :: stress_weight = 1.0d0 ! Weight of piston
      double precision :: wwr = 0.005d1
      double precision :: ww, pex, pexmat(3,3)
                                ! Weight of piston, target pressure (value, tensor)
      double precision :: pres, sgm ! Calculated pressure, stress (value)
      double precision :: sgmmat(3,3) = 0.0d0 ! Calculated stress tensor

! Elastic constants evaluation
      double precision :: econstd

! Input
      double precision :: strmat_set(3,3) ! Strain matrix

! Tolerance (MD stop policy)
! Negative value for ignorance
      double precision :: tolene = -1.0d0 ! Energy tolerance (Averaged eV per atom)
      double precision :: tolfor = -1.0d0 ! Force tolerance (Maximum F, eV/A)
      double precision :: tolst = -1.0d0 ! Stress tolerance (Mpa)
      double precision :: ene0, ene1, force0, force1 ! Values used for tolerance check
      double precision :: st0, st1 ! Values used for tolerance check
      integer :: loopflg

! Instability flag
      logical :: inst
! Voronoi polyhedron analysis, radius
      double precision :: rc_vor

      integer :: idbg
      double precision :: dbg

! Time count
      real :: atime,btime
      end module

      program main
      use variables
      use constants
      use output
      use bookkeep_param
      use rp_param
      use debug_variables
      implicit double precision (a-h,o-z)

      write (6,*)
      write (6,*) '#####################################################'
      write (6,*) '##      Molecular Dynamics Simulation Package      ##'
      write (6,*) '##         for Analysis of Solid Systems           ##'
      write (6,*) '##                 (MD-SPASS)                      ##'
      write (6,*) '##     by   Yoshi Umeno (umeno@iis.u-tokyo.ac.jp)  ##'
      write (6,*) '##     and  Takahiro Shimada                       ##'
      write (6,*) '#####################################################'
      write (6,*)

!----------------------------------------------------
!----- SETTING --------------------------------------
!----------------------------------------------------
!---INPUT VALUE---
!---PBC
! RECOMMENDED: (T T F) for 2-D, (T F F) for 1-D
      pbcx = .true.
      pbcy = .true.
      pbcz = .true.

!---STRESS & TEMPERATURE CONTROL
      sgm_set      =  0.0d6     ! Stress (Pa) (Positive is tension)
      temp_set     =  100.0d0      ! Temperature (K)
      step_ite     =  100
      algorithm    =  0
        ! 0=Velocity Verlet, NVE (Allen & Tildesley)
        ! 1=Gear's P-C, NVE (Allen & Tildesley)
        ! 2=Gear, NVT (Nose thermostat)
        ! 3=Gear, NPH (Andersen pressure control)
        ! 4=Gear, NPT (Nose and Andersen)
        ! 5=Gear, NPH-PR (Parrinello-Rahman)
        ! 6=Gear, NPT-PR (Nose and Parrinello-Rahman)
        ! 7=Stress relaxation by annealing, no atom motion
        ! 8=Stress relaxation by annealing, atom motion by verlet
        ! 107=Gear, Stress relaxation by NPH-PR & breaking
        ! 100=Configuration relaxation by NR
      vscale       =  0         ! Velocity scaling
      damper       =  0         ! Atom damping
      cellconstx   =  .false.   ! Cell constraint
      cellconsty   =  .false.   
      cellconstz   =  .false.   
      
!  Stress set
      sxx=0.0d6; syy=0.0d6; szz=0.0d6 ! <-- Edit here
      syz=0.0d6; szx=0.0d6; sxy=0.0d6 ! <-- Edit here
      sgmmat_set   = 0.0d0
      sgmmat_set(1,1) = sxx
      sgmmat_set(2,2) = syy
      sgmmat_set(3,3) = szz
      sgmmat_set(2,3) = syz !/2.0d0
      sgmmat_set(1,3) = szx !/2.0d0
      sgmmat_set(1,2) = sxy !/2.0d0
      sgmmat_set(3,2) = sgmmat_set(2,3)
      sgmmat_set(3,1) = sgmmat_set(1,3)
      sgmmat_set(2,1) = sgmmat_set(1,2)
      if (.not.pbcx) sgmmat_set(1,1)=0.0d0
      if (.not.pbcy) sgmmat_set(2,2)=0.0d0
      if (.not.pbcz) sgmmat_set(3,3)=0.0d0
      
!  Strain set
      exx=0.0d0; eyy=0.0d0; ezz=0.0d0 ! <-- Edit here
      eyz=0.0d0; ezx=0.0d0; exy=0.0d0 ! <-- Edit here
      strmat_set = 0.0d0
      strmat_set(1,1) = exx
      strmat_set(2,2) = eyy
      strmat_set(3,3) = ezz
      strmat_set(2,3) = eyz/2.0d0
      strmat_set(1,3) = ezx/2.0d0
      strmat_set(1,2) = exy/2.0d0
      strmat_set(3,2) = strmat_set(2,3)
      strmat_set(3,1) = strmat_set(1,3)
      strmat_set(2,1) = strmat_set(1,2)

!---SYSTEM SETTING
!      a=4.04959d-10
      a=3.61d-10
!      wm=am*13.0d0+bm*14.0d0
      call initconfsc(5,5,5)    ! Single crystal
      call strain               ! Apply strain matrix

!----- SETTING END ---------------------------------

!------------------------------------------------------
! Initialize variables
      call var_initialize

      call readcondition
      if (continue_md) then
         call readcont
      else
         call readconfig
!         if (ini_velocity) call initial_velocity
      endif
      hmat0 = hmat

! Set atom species
      call set_atom_species
! Weight of atom
      call atom_weight
      if (.not.continue_md) then
         if (ini_velocity) call initial_velocity
      endif
!-----
      step_start=step
      step_end=step+step_ite
      rc2=rc**2
      dt2=dt**2

!  Stress set
      sgmmat_set   = 0.0d0
      sgmmat_set(1,1) = sxx
      sgmmat_set(2,2) = syy
      sgmmat_set(3,3) = szz
      sgmmat_set(2,3) = syz !/2.0d0
      sgmmat_set(1,3) = szx !/2.0d0
      sgmmat_set(1,2) = sxy !/2.0d0
      sgmmat_set(3,2) = sgmmat_set(2,3)
      sgmmat_set(3,1) = sgmmat_set(1,3)
      sgmmat_set(2,1) = sgmmat_set(1,2)
      if (.not.pbcx) sgmmat_set(1,1)=0.0d0
      if (.not.pbcy) sgmmat_set(2,2)=0.0d0
      if (.not.pbcz) sgmmat_set(3,3)=0.0d0

      pex=-sgm_set
      pexmat=-sgmmat_set

! Weight of piston/heat-reservoir
! Recommended values:
! wwr=0.005d0   / wqr=1.5d0 for algo=2/3
! wwr=0.005d1   / wqr=1.5d1 for 4
! wwr=0.005d-35             for 5
! wwr=0.005d-33 / wqr=1.5d1 for 6
! wwr=0.005d-28             for 7,8
! wwr=0.005d-33             for 107
      if ((algorithm==2).or.(algorithm==3)) then
         wwr=0.005d0; wqr=1.5d0
      else if (algorithm==4) then
         wwr=0.005d1; wqr=1.5d1
      else if (algorithm==5) then
         wwr=0.005d-35
      else if (algorithm==6) then
         wwr=0.005d-33; wqr=1.5d1
      else if ((algorithm==7).or.(algorithm==8)) then
         wwr=0.005d-28
      else
         wwr=0.005d-33
      endif
      wwr=wwr*stress_weight
      wqr=wqr*temp_weight

      ww=wwr/a**4*wmave
      wq=wqr*wmave*a**2

! Set bookkeeping algorithm
      if (potential_func=='Tersoff') then
         bookkeep_algo = 2
      else if (potential_func=='TersoffNM') then
         bookkeep_algo = 2
      else if (potential_func=='StilWeb') then
         bookkeep_algo = 2
      else if (potential_func=='TersoffBelko') then
         bookkeep_algo = 2
      else if (potential_func=='MEAM') then
         bookkeep_algo = 2
      else
         bookkeep_algo = 1
      endif

! In the case of shell model
      if (potential_func=='Shellmodel') then
         allocate (tmprx(n/2,2),tmpry(n/2,2),tmprz(n/2,2)) 
         allocate (tmpfx(n/2,2),tmpfy(n/2,2),tmpfz(n/2,2)) 
         if ((.not.pbcx).or.(.not.pbcy).or.(.not.pbcz)) then
            print *,'For shell model calculation, only 3D periodic system is supported.'
            stop
         endif
      endif


! writing data
      call writedata

!===  1 step advance======================================

      loopflg = 0
      mdloop: do while (step < step_end)
         
      call mkq
      if ((mod(step,nbk)==0).or.(step==step_start)) then
         call out
         if (rpart) then
            call bookkeep_rp
         else
            call bookkeep
         endif
      endif
      step=step+1

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!  For debugging stress calculation
!      if ((step==1).or.(mod(step,50)==0)) then
!         call stresscheck
!      endif
!      stop
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


! For debugging the force subroutine
! (useful for writing a new force routine)
!      call forcecheck
!      call forcecheck2
!      stop

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!  For debugging force calculation:
!!!   maybe helpful when developping a subroutine
!!!   for a new potential function
      if ((debug).and.(step==1)) then
         allocate (den1(n),den2(n))
         allocate (nn(n),nnnum(n,n))
         allocate (distance(n,n))
         allocate (fcom(n,n,3))
      endif
      if ((debug).and.(step==3)) then
         den1=0.0d0; den2=0.0d0
         nn=0; nnnum=0
         call forcecheck
         stop
      endif
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!-----Force calc & numerical integral-----
      qpx=qx;qpy=qy;qpz=qz
      if (algorithm == 0) then
         call velocity_verlet_a
         call potential
         call velocity_verlet_b
         call kinetic
      else if (algorithm == 1) then
         call gear_pc_predic
         call potential
         call gear_pc_correc
         call kinetic
      else if (algorithm == 2) then
         call gear_pc_nvt_predic
         call potential
         call gear_pc_nvt_correc
      else if (algorithm == 3) then
         call gear_pc_nph_predic
         call potential
         call gear_pc_nph_correc
      else if (algorithm == 4) then
         call gear_pc_npt_predic
         call potential
         call gear_pc_npt_correc
      else if (algorithm == 5) then
         call gear_pc_nphpr_predic
         call potential
         call gear_pc_nphpr_correc
      else if (algorithm == 6) then
         call gear_pc_nptpr_predic
         call potential
         call gear_pc_nptpr_correc
      else if (algorithm == 7) then
         call potential
         call cellrlx
      else if (algorithm == 8) then
         call velocity_verlet_a
         call potential
         call velocity_verlet_b
         call kinetic
         call cellrlx
      else if (algorithm == 107) then
         call gear_pc_nphpr_predic
         call potential
         call gear_pc_nphpr_correc
         vx=0.0d0;vy=0.0d0;vz=0.0d0
         do i=1,3
            do j=1,3
               if (hamat(i,j)*hvmat(i,j)<0.0) then
                  hvmat(i,j)=0.0d0
               endif
            enddo
         enddo
!         if (step>2000) hvmat=0.0d0
      else if (algorithm == 100) then
         call potential
         call atom_relax
         call kinetic
!!! Conjugate gradient method
      else if (algorithm == 101) then
         call atom_relax_cg
         call potential
!!! FIRE (atom only)
      else if (algorithm == 102) then
         call velocity_verlet_a
         call potential
         call velocity_verlet_b
         call kinetic
         call atom_relax_fire
!!! FIRE (atom and cell) 1
      else if (algorithm == 103) then
         call gear_pc_nphpr_predic
         call potential
         call gear_pc_nphpr_correc
         call atom_relax_fire
         do i=1,3
            do j=1,3
               if (hamat(i,j)*hvmat(i,j)<0.0) then
                  hvmat(i,j)=0.0d0
               endif
            enddo
         enddo
!!! FIRE (atom and cell) 2 (not robust, not efficient ?)
      else if (algorithm == 104) then
         call velocity_verlet_a
         call potential
         call velocity_verlet_b
         call kinetic
         call atom_relax_fire
         call cellrlx
      else
         print *,'Algorithm is not specified.'
         stop
      endif
! Atom motion constraint
      do i=1,n
         if (mfx(i)=='F') then;qx(i)=qpx(i);vx(i)=0.0d0;endif
         if (mfy(i)=='F') then;qy(i)=qpy(i);vy(i)=0.0d0;endif
         if (mfz(i)=='F') then;qz(i)=qpz(i);vz(i)=0.0d0;endif
      enddo

! Total drift check
!      print *,'Total drift energy (eV)',real(totaldrift()/ev)

      if (vscale /= 0) then
         if (mod(step,vscale) == 0) then
            call velocity_scaling
         endif
      endif

      if (damper /= 0) then
         if (mod(step,damper) == 0) then
            call atom_damping
         endif
      endif

      if ((cellconstx).or.(cellconsty).or.(cellconstz)) then
         call cellconstraint
      endif

!---output---
      pres=(enkin*2.0d0+virx+viry+virz)/v/3.0d0
      sgm=-pres
      call calcstresstensor
      temp=enkin*2.0d0/3.0d0/dble(n)/bl
      call writedata
!--- MD end criterion
      if ((tolene>0.0).or.(step>=step_end-2)) then
         ene0=ene1
         ene1=enpot/ev/n
         if (step-step_start>1) then
            if (abs(ene1-ene0)<tolene) loopflg=1
         endif
      endif
      if ((tolst>0.0).or.(step>=step_end-2)) then
         st0=st1
         st1=max(abs(maxval(sgmmat)),abs(minval(sgmmat)))
         if (step-step_start>1) then
            if (abs(st1-st0)<tolst) loopflg=3
         endif
      endif
      if ((tolfor>0.0).or.(step==step_end)) then
         force1=-1.0
         do i=1,n
            fvec=fx(i)**2+fy(i)**2+fz(i)**2
            if (force1<fvec) then
               force1=fvec
               iii=i
            endif
         enddo
         force1=dsqrt(force1)/ev*ang
         if (force1<tolfor) loopflg=2
         force0=force1
      endif
      if (loopflg>0) exit
!---  loop---
      enddo mdloop

      if (loopflg==0) then
         print *,'MD ended: maximum step number is reached'
      else if (loopflg==1) then
         print *,'MD ended: energy is converged'
      else if (loopflg==2) then
         print *,'MD ended: force is converged'
      else if (loopflg==3) then
         print *,'MD ended: stress is converged'
      endif
      print *,'Energy difference:',abs(ene1-ene0),' eV per atom'
      print *,'Maximum force:',force1,'eV/ang'
      call writedata

! Elastic constants (numerical evaluation): use with care
      if (econstd>0.0) then
         call calcelastic
      endif

! Instability check
      if (inst) then
         call mkq
         call out
         call bookkeep
         call instability_cellonly
         call instability_atomonly
      endif

! Voronoi polyhedron analysis
      if (rc_vor>0.0) then
         call voronoi
      endif

      print *
      print *,'## MD-SPASS END ##'
      print *,'Total warning(s):',iwarn
      atime=etime(btime)
      print *,'Total calculation time: ',btime,'  sec'
      print *

      end
