!
! EMT potential force and potential (and stress term) calculation
!  for Copper
!
! H.Haekkinen and M.Manninen
!  J.Phys.: Condens. Matter 1 (1989) 9765-9777
!  "The effective-medium theory beyond the nearest-neighbour interaction"
!  New bookkeep algorithm
!
      subroutine emt_HM_JP1989
      use variables
      use constants
      use bookkeep_param
      use debug_variables
      implicit double precision (a-h,o-z)
      double precision :: emte0, emte2, emte3 &
      ,emtn0, emteta, emts0, emtgam1, emtbet, emteta2 &
      ,emtrc, emta, emtalp, emtgam2
      double precision :: gamma1,gamma2,emteta1
      double precision :: ki
      integer :: istat
      double precision :: emtnb(n),emts(n) ! bar{n_i}, s_i
      double precision :: gamma1_calc,gamma2_calc

      istat=1

      if ((asp(1)/='Cu').and.(asp(1)/='cu')) then
         print *,'emt_HM_JP1989 is prepared only for copper'
         print *, asp(1:2),' is not supported'
         stop
      endif

!   EMT PARAMETER (for Copper, p.9772 in the paper)
      emtn0=0.0115d0/(bohr**3)
      emts0=2.5776d0*bohr
      emteta=2.50d0/bohr
      emte0=-3.56d0*ev
      emte2=1.35d0*ev
      emte3=-0.237d0*ev
      emtalp=1490.0d0*ev*bohr**3
      emteta1=0.25d0/bohr
      emtbet=(16.0d0*pi/3.0d0)**(1.0d0/3.0d0)/sqrt(2.0d0)
      emteta2=(emteta+emteta1)/emtbet
      emtrc=8.648d-10
      emtrc2=emtrc**2
      rc=emtrc
      rc2=emtrc2

!      emtgam1=1.043d0
!      emtgam2=1.069d0
!      emta=-10.0d10

!   EMT PARAMETER END

!      emtsig1=0.0d0
!      emtsig2=0.0d0

!   Virial term reset
      virx=0.0d0; viry=0.0d0; virz=0.0d0
      dmat=0.0d0

! First loop
! to determine emts (s_i)
      gamma1=gamma1_calc(emts0,emtbet,emteta2) ! Initial value (using s=s0)
      emts=0.0d0
      emtnb=0.0d0
      enpoteach=0.0d0
      kns=0
      do 4200 k=1,n
         kn=knum(k)
         if (kn.eq.0) go to 4200
         do 4300 jk=1,kn
            kns=kns+1
            j=idpt(kns,1)
            ix=idpt(kns,2); iy=idpt(kns,3); iz=idpt(kns,4)
            xx=rx(k)-rx(j)
            yy=ry(k)-ry(j)
            zz=rz(k)-rz(j)
            call calc_replica(xx,yy,zz,rc,ix,iy,iz,istat)
            if (istat == 1) then
               r2=xx**2+yy**2+zz**2
               if (r2.lt.rc2) then
                  r=sqrt(r2)
                  emts(k)=emts(k)+dexp(-emteta2*r)
                  emtnb(k)=emtnb(k)+dexp(-emteta2*(r-emtbet*emts0))
                  if (j/=k) then
                     emts(j)=emts(j)+dexp(-emteta2*r)
                     emtnb(j)=emtnb(j)+dexp(-emteta2*(r-emtbet*emts0))
                  endif
               endif
            endif
 4300    continue
 4200 continue

      emts=-1.0d0/(emtbet*emteta2)*dlog(emts/12.0d0/gamma1)
! bar{s} (average of s_i), gamma_1, gamma_2
      emtsb=sum(emts)/dble(n)
      gamma1=gamma1_calc(emtsb,emtbet,emteta2)
      gamma2=gamma2_calc(emtsb,emteta)

      emtnb=emtn0*(emtnb/12.0d0/gamma1)**(emteta/(emteta+emteta1))

!     Calculation of potential energy
      enpoteach=emte0+emte2*(emtnb/emtn0-1.0d0)**2+emte3*(emtnb/emtn0-1.0d0)**3 &
      +emtalp*emtnb

!   Second Loop: Force calc
      fx(1:n)=0.0d0
      fy(1:n)=0.0d0
      fz(1:n)=0.0d0
      const_n=-emtn0*emteta*emteta2*dexp(emteta2*emtbet*emts0) &
      /12.0d0/gamma1/(emteta+emteta1)
      
      const_v=emtalp*emtn0*emteta*dexp(emteta*emts0)/(6.0d0*emtbet*gamma2)
!   For extra pressure term
      vijsum=0.0d0
!
      kns=0
      do 4500 i=1,n
         kn=knum(i)
         if (kn.eq.0) go to 4500
         do 4600 jk=1,kn
            kns=kns+1
            j=idpt(kns,1)
            ix=idpt(kns,2); iy=idpt(kns,3); iz=idpt(kns,4)
            xx=rx(i)-rx(j)
            yy=ry(i)-ry(j)
            zz=rz(i)-rz(j)
            call calc_replica(xx,yy,zz,rc,ix,iy,iz,istat)
            if (istat == 1) then
               r2=xx**2+yy**2+zz**2
               if (r2.lt.rc2) then
                  r=sqrt(r2)
                  vij=-const_v*emtbet/emteta*dexp(-emteta*r/emtbet)
                  enpoteach(i)=enpoteach(i)+vij/2.0d0
                  vijsum=vijsum+vij/2.0d0
                  if (j/=i) then
                     enpoteach(j)=enpoteach(j)+vij/2.0d0
                     vijsum=vijsum+vij/2.0d0
                  endif
                  
! E'_N
                  den_i=2.0d0*emte2/emtn0*(emtnb(i)/emtn0-1.0d0) &
                  +3.0d0*emte3/emtn0*(emtnb(i)/emtn0-1.0d0)**2+emtalp
                  den_j=2.0d0*emte2/emtn0*(emtnb(j)/emtn0-1.0d0) &
                  +3.0d0*emte3/emtn0*(emtnb(j)/emtn0-1.0d0)**2+emtalp
! bar{n}'
                  dn_i=const_n*((emtnb(i)/emtn0)**(-emteta1/emteta)) &
                  *dexp(-emteta2*r)
                  dn_j=const_n*((emtnb(j)/emtn0)**(-emteta1/emteta)) &
                  *dexp(-emteta2*r)
! V'
                  dv=const_v*dexp(-emteta*r/emtbet)

                  f=-(den_i*dn_i+den_j*dn_j+dv)

                  ki=f/r
                  if (j==i) ki=ki/2.0
                  dxx1=ki*xx
                  dyy1=ki*yy
                  dzz1=ki*zz
                  fx(i)=fx(i)+dxx1
                  fy(i)=fy(i)+dyy1
                  fz(i)=fz(i)+dzz1
                  fx(j)=fx(j)-dxx1
                  fy(j)=fy(j)-dyy1
                  fz(j)=fz(j)-dzz1
                  dmat(1,1)=dmat(1,1)+ki*xx**2
                  dmat(2,1)=dmat(2,1)+ki*xx*yy
                  dmat(3,1)=dmat(3,1)+ki*xx*zz
                  dmat(1,2)=dmat(1,2)+ki*xx*yy
                  dmat(2,2)=dmat(2,2)+ki*yy**2
                  dmat(3,2)=dmat(3,2)+ki*yy*zz
                  dmat(1,3)=dmat(1,3)+ki*xx*zz
                  dmat(2,3)=dmat(2,3)+ki*yy*zz
                  dmat(3,3)=dmat(3,3)+ki*zz**2
                  virx=virx+xx*dxx1
                  viry=viry+yy*dyy1
                  virz=virz+zz*dzz1
               endif
            endif
 4600    continue
 4500 continue

! Extra pressure term
! E'_N*bar{n} sum
      denbsum=0.0d0
      do i=1,n
         den_i=2.0d0*emte2/emtn0*(emtnb(i)/emtn0-1.0d0) &
         +3.0d0*emte3/emtn0*(emtnb(i)/emtn0-1.0d0)**2+emtalp
         denbsum=denbsum+den_i*emtnb(i)
      enddo
! gamma1', gamma2'
      diff_gamma1=diff_gamma1_calc(emtsb,emtbet,emteta2)
      diff_gamma2=diff_gamma2_calc(emtsb,emteta)
! Extra pressure term (note that Eq.(28) in Ref is wrong)
      p_gamma= (-denbsum*emteta*diff_gamma1/((emteta+emteta1)*gamma1) &
      -diff_gamma2*vijsum/gamma2) /(-4.0d0*pi*dble(n)*emtsb**2)
!      print *,'Extra pressure term:',p_gamma/mpa
      p_gamma=p_gamma*v
      do i=1,3
         dmat(i,i)=dmat(i,i)+p_gamma
      enddo
      virx=virx+p_gamma; viry=viry+p_gamma; virz=virz+p_gamma

!---Etot---
      enpot=0.0d0
      do i=1,n
         enpot=enpot+enpoteach(i)
      enddo

!      write (10,*) 'EMT force calculation end.'
      end

! gamma_1 (Eq.(17))
      function gamma1_calc(s,beta,eta2)
      double precision :: gamma1_calc
      double precision, intent(in) :: beta,eta2
      double precision, intent(in) :: s
      double precision :: pre_calc
      pre_calc=beta*eta2*s
      gamma1_calc=1.0d0 &
      +0.5d0*dexp((1.0d0-dsqrt(2.0d0))*pre_calc) &
      +2.0d0*dexp((1.0d0-dsqrt(3.0d0))*pre_calc) &
      +dexp(-1.0d0*pre_calc) &
      +2.0d0*dexp((1.0d0-dsqrt(5.0d0))*pre_calc) &
      +2.0d0*dexp((1.0d0-dsqrt(6.0d0))*pre_calc)/3.0d0 &
      +4.0d0*dexp((1.0d0-dsqrt(7.0d0))*pre_calc) &
      +dexp((1.0d0-2.0d0*dsqrt(2.0d0))*pre_calc)/2.0d0 &
      +3.0d0*dexp(-2.0d0*pre_calc) &
      +2.0d0*dexp((1.0d0-dsqrt(10.0d0))*pre_calc) &
      +2.0d0*dexp((1.0d0-dsqrt(11.0d0))*pre_calc) &
      +2.0d0*dexp((1.0d0-dsqrt(12.0d0))*pre_calc)
      end function gamma1_calc
      
! gamma_2 (Eq.(21))
      function gamma2_calc(s,eta)
      double precision :: gamma2_calc
      double precision, intent(in) :: eta
      double precision, intent(in) :: s
      double precision :: pre_calc
      pre_calc=eta*s
      gamma2_calc=1.0d0 &
      +0.5d0*dexp((1.0d0-dsqrt(2.0d0))*pre_calc) &
      +2.0d0*dexp((1.0d0-dsqrt(3.0d0))*pre_calc) &
      +dexp(-1.0d0*pre_calc) &
      +2.0d0*dexp((1.0d0-dsqrt(5.0d0))*pre_calc) &
      +2.0d0*dexp((1.0d0-dsqrt(6.0d0))*pre_calc)/3.0d0 &
      +4.0d0*dexp((1.0d0-dsqrt(7.0d0))*pre_calc) &
      +dexp((1.0d0-2.0d0*dsqrt(2.0d0))*pre_calc)/2.0d0 &
      +3.0d0*dexp(-2.0d0*pre_calc) &
      +2.0d0*dexp((1.0d0-dsqrt(10.0d0))*pre_calc) &
      +2.0d0*dexp((1.0d0-dsqrt(11.0d0))*pre_calc) &
      +2.0d0*dexp((1.0d0-dsqrt(12.0d0))*pre_calc)
      end function gamma2_calc
      
! d(gamma_1)/d(s-bar)
      function diff_gamma1_calc(s,beta,eta2)
      double precision :: diff_gamma1_calc
      double precision, intent(in) ::beta,eta2
      double precision, intent(in) ::s
      double precision :: pre_calc
      double precision :: pre_calc2
      
      pre_calc=beta*eta2*s
      pre_calc2=beta*eta2
      diff_gamma1_calc=0.0d0 &
      +0.5d0*(1.0d0-dsqrt(2.0d0))*pre_calc2*dexp((1.0d0-dsqrt(2.0d0))*pre_calc) &
      +2.0d0*(1.0d0-dsqrt(3.0d0))*pre_calc2*dexp((1.0d0-dsqrt(3.0d0))*pre_calc) &
      -pre_calc2*dexp(-1.0d0*pre_calc) &
      +2.0d0*(1.0d0-dsqrt(5.0d0))*pre_calc2*dexp((1.0d0-dsqrt(5.0d0))*pre_calc) &
      +2.0d0*(1.0d0-dsqrt(6.0d0))*pre_calc2*dexp((1.0d0-dsqrt(6.0d0))*pre_calc)/3.0d0 &
      +4.0d0*(1.0d0-dsqrt(7.0d0))*pre_calc2*dexp((1.0d0-dsqrt(7.0d0))*pre_calc) &
      +(1.0d0-2.0d0*dsqrt(2.0d0))*pre_calc2*dexp((1.0d0-2.0d0*dsqrt(2.0d0))*pre_calc)/2.0d0 &
      -6.0d0*pre_calc2*dexp(-2.0d0*pre_calc) &
      +2.0d0*(1.0d0-dsqrt(10.0d0))*pre_calc2*dexp((1.0d0-dsqrt(10.0d0))*pre_calc) &
      +2.0d0*(1.0d0-dsqrt(11.0d0))*pre_calc2*dexp((1.0d0-dsqrt(11.0d0))*pre_calc) &
      +2.0d0*(1.0d0-dsqrt(12.0d0))*pre_calc2*dexp((1.0d0-dsqrt(12.0d0))*pre_calc)
      end function diff_gamma1_calc

! d(gamma_2)/d(s-bar)
      function diff_gamma2_calc(s,eta)
      double precision :: diff_gamma2_calc
      double precision, intent(in) :: eta
      double precision, intent(in) :: s
      double precision :: pre_calc
      
      pre_calc=eta*s
      diff_gamma2_calc=0.0d0 &
      +0.5d0*(1.0d0-dsqrt(2.0d0))*eta*dexp((1.0d0-dsqrt(2.0d0))*pre_calc) &
      +2.0d0*(1.0d0-dsqrt(3.0d0))*eta*dexp((1.0d0-dsqrt(3.0d0))*pre_calc) &
      -eta*dexp(-1.0d0*pre_calc)&
      +2.0d0*(1.0d0-dsqrt(5.0d0))*eta*dexp((1.0d0-dsqrt(5.0d0))*pre_calc) &
      +2.0d0*(1.0d0-dsqrt(6.0d0))*eta*dexp((1.0d0-dsqrt(6.0d0))*pre_calc)/3.0d0 &
      +4.0d0*(1.0d0-dsqrt(7.0d0))*eta*dexp((1.0d0-dsqrt(7.0d0))*pre_calc) &
      +(1.0d0-2.0d0*dsqrt(2.0d0))*eta*dexp((1.0d0-2.0d0*dsqrt(2.0d0))*pre_calc)/2.0d0 &
      -6.0d0*eta*dexp(-2.0d0*pre_calc) &
      +2.0d0*(1.0d0-dsqrt(10.0d0))*eta*dexp((1.0d0-dsqrt(10.0d0))*pre_calc) &
      +2.0d0*(1.0d0-dsqrt(11.0d0))*eta*dexp((1.0d0-dsqrt(11.0d0))*pre_calc) &
      +2.0d0*(1.0d0-dsqrt(12.0d0))*eta*dexp((1.0d0-dsqrt(12.0d0))*pre_calc)
      end function diff_gamma2_calc

