!
!   Modified EAM potential
!   for Silicon
!    H. Huang et al.
!    "Molecular dynamics determination of defect energies ..."
!    Modelling Simul. Mater. Sci. Eng., Vol. 3, pp.615-627 (1995)
!
!   calculation of force, total energy, stress
!

      module meam_HuangSiC_function
        use constants
        double precision :: aa(2), rho0(2), rr0(3), ee0(3), alp(3)
        double precision :: bet0(2), bet1(2), bet2(2), bet3(2)
        double precision :: t00(2), t01(2), t02(2), t03(2)
        double precision :: ccmin(3), ccmax(3)
        double precision :: ss0, ss1, ss2, ss3, zz0,zz1
        double precision, allocatable :: rhob(:), rhobp(:,:), rhos(:)
        double precision, allocatable :: t(:,:), ttb(:,:)
        double precision :: tmpa(3),tmpb(3,3),tmpc(3,3,3),tmpbb
        double precision, allocatable :: rs(:),xrs(:,:),xxrs(:,:,:),xxxrs(:,:,:,:)
        double precision, allocatable :: ss(:,:)
!        double precision, allocatable :: ds_x(:,:,:),ds_y(:,:,:),ds_z(:,:,:)
        double precision, allocatable :: ds_x(:,:),ds_y(:,:),ds_z(:,:)
        double precision, allocatable :: drhob_x(:,:),drhob_y(:,:),drhob_z(:,:)
        double precision :: xij(3), dxijdrix(3), dxijdriy(3), dxijdriz(3)
        double precision :: xji(3), dxjidrix(3), dxjidriy(3), dxjidriz(3)
        double precision :: xjk(3)
        double precision :: rij, rji, rik, rki, rjk, rkj
        double precision :: rij2,rji2,rik2,rki2,rjk2,rkj2
! stress
        double precision, allocatable :: ds_e(:,:,:,:), drhob_e(:,:,:)
        double precision :: dttide(0:3,3,3), dtide(0:3,3,3), drhobi0de(3,3)
        double precision :: drhobi1de(3,3), drhobi2de(3,3), drhobi3de(3,3), drhoi0de(3,3)
! for reference background charge density function
        double precision, allocatable :: rhobrefx(:),rhobrefy(:),rhobrefc(:,:)
         integer, parameter :: mesh=1000000
        double precision :: rhobref_Si(mesh),drhobref_Si(mesh)
        double precision :: rhobref_C(mesh),drhobref_C(mesh)
        double precision, allocatable :: rhobrefxn(:),spd2(:)
        dimension DF(2), IOPT(2), EC(4)
        double precision :: refxmin,refxmax
        logical, save :: meam_HuangSiC_setup_done = .false.
      contains
        double precision function ee(i,j,r)
          double precision :: r
          integer :: i,j,k,ispc,jspc,ii
          ispc=tsp(asp(i)); jspc=tsp(asp(j))
          ii=3; if (ispc==jspc) ii=jspc
          ee=-ee0(ii)*(1.0d0+alp(ii)*(r/rr0(ii)-1.0d0)) &
               * exp(-alp(ii)*(r/rr0(ii)-1.0d0))
        end function ee
        double precision function dee(i,j,r)
          double precision :: r
          integer :: i,j,k,ispc,jspc,ii
          ispc=tsp(asp(i)); jspc=tsp(asp(j))
          ii=3; if (ispc==jspc) ii=jspc
          dee=ee0(ii)*alp(ii)**2/rr0(ii) * ((r/rr0(ii)-1.0d0)) &
               * exp(-alp(ii)*(r/rr0(ii)-1.0d0))
        end function dee
        double precision function ff(i,x)
          double precision :: x
          integer :: i,ispc
          ispc=tsp(asp(i))
          ff=aa(ispc)*ee0(ispc)*x*log(x)
        end function ff
        double precision function dff(i,x)
          double precision :: x
          integer :: i,ispc
          ispc=tsp(asp(i))
          dff=aa(ispc)*ee0(ispc)*(1.0d0+log(x))
        end function dff
        double precision function rhoa(i,j,k,r)
          double precision :: r
          integer :: i,j,k,ispc,jspc,ii
          ispc=tsp(asp(i)); jspc=tsp(asp(j))
          ii=3; if (ispc==jspc) ii=jspc
          if (k==0) then
             rhoa=rho0(ispc)*exp(-bet0(ispc)*(r/rr0(ii)-1.0d0))
          else if (k==1) then
             rhoa=rho0(ispc)*exp(-bet1(ispc)*(r/rr0(ii)-1.0d0))
          else if (k==2) then
             rhoa=rho0(ispc)*exp(-bet2(ispc)*(r/rr0(ii)-1.0d0))
          else if (k==3) then
             rhoa=rho0(ispc)*exp(-bet3(ispc)*(r/rr0(ii)-1.0d0))
          endif
        end function rhoa
        double precision function drhoa(i,j,k,r)
          double precision :: r
          integer :: i,j,k,ispc,jspc,ii
          ispc=tsp(asp(i)); jspc=tsp(asp(j))
          ii=3; if (ispc==jspc) ii=jspc
          if (k==0) then
             drhoa=-bet0(ispc)/rr0(ii)*rhoa(i,j,k,r)
          else if (k==1) then
             drhoa=-bet1(ispc)/rr0(ii)*rhoa(i,j,k,r)
          else if (k==2) then
             drhoa=-bet2(ispc)/rr0(ii)*rhoa(i,j,k,r)
          else if (k==3) then
             drhoa=-bet3(ispc)/rr0(ii)*rhoa(i,j,k,r)
          endif
        end function drhoa
        integer function tsp(species)
          character(2) :: species
          if (species=='Si') tsp=1
          if (species=='C ') tsp=2
        end function tsp
        double precision function drde(xx,yy,zz,r,ia,ib)
          double precision :: xx,yy,zz,r
          integer :: ia,ib
          if (ia==1) ra=xx; if (ib==1) rb=xx
          if (ia==2) ra=yy; if (ib==2) rb=yy
          if (ia==3) ra=zz; if (ib==3) rb=zz
          drde=ra*rb/r
        end function drde
        double precision function dxde(rix,riy,riz,rjx,rjy,rjz,rij,ial,ia,ib)
          double precision :: rix,riy,riz,rjx,rjy,rjz,rij,x1,x2
          integer :: ial,ia,ib
          if (ial==ib) then
             if (ia==1) x1=(rjx-rix)*rij
             if (ia==2) x1=(rjy-riy)*rij
             if (ia==3) x1=(rjz-riz)*rij
          else
             x1=0.0d0
          endif
          if (ial==1) x2=(rjx-rix)*drde(rjx-rix,rjy-riy,rjz-riz,rij,ia,ib)
          if (ial==2) x2=(rjy-riy)*drde(rjx-rix,rjy-riy,rjz-riz,rij,ia,ib)
          if (ial==3) x2=(rjz-riz)*drde(rjx-rix,rjy-riy,rjz-riz,rij,ia,ib)
          dxde=(x1-x2)/rij**2
        end function dxde
      end module

      subroutine meam_HuangSiC
      use meam_HuangSiC_function
      use variables
      use constants
      use bookkeep_param
      use debug_variables
      use bookkeep_param
      use debug_variables
      implicit double precision (a-h,o-z)
!      implicit none
!      double precision :: strmat_store(3,3),hmat_store(3,3)
!      double precision,allocatable :: rx_store(:),ry_store(:),rz_store(:)
      double precision :: inf_str
      double precision :: ki

! Reference function setup
      if (.not.meam_HuangSiC_setup_done) then
         call meam_HuangSiC_setup
      endif

      if (bookkeep_algo /= 2) then
         stop 'Error in tersoff.F: Bookkeep algo is not DOUBLE'
      endif

      if (.not.(allocated(rhob))) then
         allocate (rhob(n),rhobp(n,0:3),rhos(n),t(n,0:3),ttb(n,0:3))
         allocate (rs(n),xrs(n,3),xxrs(n,3,3),xxxrs(n,3,3,3))
         allocate (ss(n,n))
!         allocate (ds_x(n,n,n),ds_y(n,n,n),ds_z(n,n,n))
         allocate (ds_x(n,n),ds_y(n,n),ds_z(n,n))
         allocate (ds_e(n,n,3,3),drhob_e(n,3,3))
         allocate (drhob_x(n,n),drhob_y(n,n),drhob_z(n,n))
      endif
      
!--
      pi=4.0d0*atan(1.0d0)
      eps=1.0d-30; eps0=1.0d-10
      ev2j=1.6021892d-19        !eV -> J
      j2ev=1.0d0/ev2j           !J -> eV
!  PARAMETER
!  For Si
      aa(1)=   1.0d0
      rho0(1)= 1.0d0
      rr0(1)=  2.3517d-10
      ee0(1)=  4.63d0*ev2j
      alp(1)=  4.87d0
      bet0(1)= 4.8d0
      bet1(1)= 4.8d0
      bet2(1)= 4.8d0
      bet3(1)= 4.8d0
      t00(1)=  1.0d0
      t01(1)=  3.3d0
      t02(1)=  5.105d0
      t03(1)= -0.8d0
      ccmin(1)=2.0d0
      ccmax(1)=2.8d0
!  For C
      aa(2)=   1.0d0
      rho0(2)= 2.25d0
      rr0(2)=  1.5446d-10
      ee0(2)=  7.37d0*ev2j
      alp(2)=  4.38d0
      bet0(2)= 4.1d0
      bet1(2)= 4.2d0
      bet2(2)= 5.0d0
      bet3(2)= 3.0d0
      t00(2)=  1.0d0
      t01(2)=  5.0d0
      t02(2)=  9.34d0
      t03(2)= -1.0d0
      ccmin(2)=2.0d0
      ccmax(2)=2.8d0
!  For Si-C
      rr0(3)=  1.8878d-10
      ee0(3)=  6.433d0*ev2j
      alp(3)=  4.37d0
      ccmin(3)=2.0d0
      ccmax(3)=4.0d0

! Geometry factors
      ss0=1.0d0
      ss1=0.0d0
      ss2=0.0d0
      ss3=32.0d0/9.0d0
      zz0=4.0d0
      zz1=4.0d0

!--
      istat=1

!      rc=3.2d-10
      rc=2.5d-10
      rc2=rc**2
      frc2=frc**2
      fac=1.0

!   Virial term reset
      virx=0.0d0; viry=0.0d0; virz=0.0d0
      dmat=0.0d0
      satom=0.0d0               ! Atomic stress

!   Loop
      enpot=0.0d0
      enpoteach=0.0d0
      fx(1:n)=0.0d0
      fy(1:n)=0.0d0
      fz(1:n)=0.0d0

      rhob=0.0d0; rhobp=0.0d0; rhos=0.0d0; ttb=0.0d0
      ss=0.0d0; ds_x=0.0d0;ds_y=0.0d0;ds_z=0.0d0
      drhob_x=0.0d0;drhob_y=0.0d0;drhob_z=0.0d0
      ds_e=0.0d0

! 1st set of loops
      xx=0.0d0; xxx=0.0d0
      kns=0
      iloop1: do i=1,n
         rix=rx(i)
         riy=ry(i)
         riz=rz(i)
         kn=knum(i)
         if (kn.eq.0) cycle
         kns0=kns
         tmpa=0.0d0;tmpb=0.0d0;tmpc=0.0d0;tmpbb=0.0d0
         jloop1: do jcount=1,kn
            kns=kns+1
            j=idpt(kns,1)
            ix=idpt(kns,2); iy=idpt(kns,3); iz=idpt(kns,4)
            xx=rx(i)-rx(j); yy=ry(i)-ry(j); zz=rz(i)-rz(j)
            call calc_replica(xx,yy,zz,rc,ix,iy,iz,istat)
            rjx=rx(i)-xx; rjy=ry(i)-yy; rjz=rz(i)-zz
            rij2=(rjx-rix)**2+(rjy-riy)**2+(rjz-riz)**2
            if (rij2.le.eps) go to 201
            if (rij2.ge.rc2) go to 201
            rij=sqrt(rij2)
            rji=rij; rji2=rij2
            
            sji=1.0d0;

            knsk=kns0
            kloop1: do kcount=1,kn   ! <to calculate S_ij>
               knsk=knsk+1
               k=idpt(knsk,1)
               ix=idpt(knsk,2); iy=idpt(knsk,3); iz=idpt(knsk,4)
               xx=rx(i)-rx(k); yy=ry(i)-ry(k); zz=rz(i)-rz(k)
               call calc_replica(xx,yy,zz,rc,ix,iy,iz,istat)
               rkx=rx(i)-xx; rky=ry(i)-yy; rkz=rz(i)-zz
               rik2=(rkx-rix)**2+(rky-riy)**2+(rkz-riz)**2
               
               if (rik2.le.eps) go to 301
!               if (rik2.ge.rc2) go to 301
               rik=sqrt(rik2)
               rki=rik; rki2=rik2
               rjk2=(rkx-rjx)**2+(rky-rjy)**2+(rkz-rjz)**2
               if (rjk2.le.eps) go to 301
               rjk=sqrt(rjk2)
               rkj=rjk; rkj2=rjk2
               
               xxik=(rik2/rij2); xxjk=(rjk2/rij2)
               cos1=(rik2+rij2-rjk2)/(2.0*rik*rij)
               cos2=(rij2+rjk2-rik2)/(2.0*rij*rjk)
               if ((abs(xxik-xxjk)-1.0<eps).and.(abs(xxik-xxjk)-1.0>-eps)) then
                  cc=-1.0d0
               else if ((cos1<=0.0).or.(cos2<=0.0)) then
                  cc=-1.0d0
               else
                  cc=( 2.0d0*(xxik+xxjk) - (xxik-xxjk)**2 - 1.0d0 ) / ( 1.0d0-(xxik-xxjk)**2 )
               endif
               if ((tsp(asp(i))==1).and.(tsp(asp(j))==1)) then
                  cmin=ccmin(1); cmax=ccmax(1)
               else if ((tsp(asp(i))==2).and.(tsp(asp(j))==2)) then
                  cmin=ccmin(2); cmax=ccmax(2)
               else
                  cmin=ccmin(3); cmax=ccmax(3)
               endif
               if (cc<=0.0) then
                  sjki=1.0d0
               else if (cc<=cmin) then
                  sjki=0.0d0
               else if (cc<=cmax) then
                  sjki=(1.0d0-((cmax-cc)/(cmax-cmin))**4)**2
               else
                  sjki=1.0d0
               endif
               sji=sji*sjki
301            continue
            end do kloop1
            sij=sji; ss(j,i)=sji; ss(i,j)=sij

            ispc=tsp(asp(i)); jspc=tsp(asp(j))

! bar{T}_i^(k) (ttb)
            ttb(i,0)=ttb(i,0)+t00(jspc)*rhoa(j,i,0,rji)*sji
            ttb(i,1)=ttb(i,1)+t01(jspc)*rhoa(j,i,0,rji)*sji
            ttb(i,2)=ttb(i,2)+t02(jspc)*rhoa(j,i,0,rji)*sji
            ttb(i,3)=ttb(i,3)+t03(jspc)*rhoa(j,i,0,rji)*sji

! bar{rho}_i^(k)
            rhobp(i,0)=rhobp(i,0)+rhoa(j,i,0,rji)*sji
            xij(1)=(rjx-rix)/rij; xij(2)=(rjy-riy)/rij; xij(3)=(rjz-riz)/rij
            do ia=1,3
               tmpa(ia)=tmpa(ia)+xij(ia)*rhoa(j,i,1,rji)*sji
               do ip=1,3
                  tmpb(ia,ip)=tmpb(ia,ip)+xij(ia)*xij(ip)*rhoa(j,i,2,rji)*sji
                  do iq=1,3
                     tmpc(ia,ip,iq)=tmpc(ia,ip,iq)+xij(ia)*xij(ip)*xij(iq)*rhoa(j,i,3,rji)*sji
                  enddo
               enddo
            enddo
            tmpbb=tmpbb+rhoa(j,i,2,rji)*sji
201         continue
         end do jloop1
         rs(i)=tmpbb
         do ia=1,3
            xrs(i,ia)=tmpa(ia)
            do ip=1,3
               xxrs(i,ia,ip)=tmpb(ia,ip)
               do iq=1,3
                  xxxrs(i,ia,ip,iq)=tmpc(ia,ip,iq)
               enddo
            enddo
         enddo
         
         do ia=1,3
            rhobp(i,1)=rhobp(i,1)+tmpa(ia)**2
            do ip=1,3
               rhobp(i,2)=rhobp(i,2)+tmpb(ia,ip)**2
               do iq=1,3
                  rhobp(i,3)=rhobp(i,3)+tmpc(ia,ip,iq)**2
               enddo
            enddo
         enddo
         rhobp(i,1)=sqrt(rhobp(i,1))
         if (rhobp(i,2)-rs(i)**2/3.0d0>0.0) then
            rhobp(i,2)=sqrt(rhobp(i,2)-rs(i)**2/3.0d0)
         else
            rhobp(i,2)=0.0d0
         endif
         rhobp(i,3)=sqrt(rhobp(i,3))

! t_i^(k)
         if (rhobp(i,0)/=0.0) then
            t(i,0)=ttb(i,0)/rhobp(i,0)
            t(i,1)=ttb(i,1)/rhobp(i,0)
            t(i,2)=ttb(i,2)/rhobp(i,0)
            t(i,3)=ttb(i,3)/rhobp(i,0)
         else
            t(i,0)=0.0d0;t(i,1)=0.0d0;t(i,2)=0.0d0;t(i,3)=0.0d0
         endif

         rhos(i)=rho0(ispc)*zz0*exp(-0.5d0*(ss1*t(i,1)+ss2*t(i,2)+ss3*t(i,3))/zz0**2)

         if (rhobp(i,0)/=0.0) then
            rhob(i)=rhobp(i,0)/zz1/rhos(i)*exp(-0.5d0*( &
                 ((rhobp(i,1)*t(i,1))**2 + (rhobp(i,2)*t(i,2))**2 + (rhobp(i,3)*t(i,3))**2) &
                 /rhobp(i,0)**2 ))
         else
            rhob(i)=0.0d0
         endif
      end do iloop1

! 2nd set of loops
      kns=0
      iloop2: do i=1,n
         rix=rx(i); riy=ry(i); riz=rz(i)
         kn=knum(i)
         if (kn.eq.0) cycle
         kns0=kns
         tmpa=0.0d0;tmpb=0.0d0;tmpc=0.0d0;tmpbb=0.0d0
         dtti0drix=0.0d0;dtti1drix=0.0d0;dtti2drix=0.0d0;dtti3drix=0.0d0
         dtti0driy=0.0d0;dtti1driy=0.0d0;dtti2driy=0.0d0;dtti3driy=0.0d0
         dtti0driz=0.0d0;dtti1driz=0.0d0;dtti2driz=0.0d0;dtti3driz=0.0d0
         drhobi0drix=0.0d0;drhobi0driy=0.0d0;drhobi0driz=0.0d0
         drhobi1drix=0.0d0;drhobi1driy=0.0d0;drhobi1driz=0.0d0
         drhobi2drix=0.0d0;drhobi2driy=0.0d0;drhobi2driz=0.0d0
         drhobi3drix=0.0d0;drhobi3driy=0.0d0;drhobi3driz=0.0d0
         dttide=0.0d0; drhobi0de=0.0d0;drhobi1de=0.0d0;drhobi2de=0.0d0;drhobi3de=0.0d0
         ds_x=0.0d0;ds_y=0.0d0;ds_z=0.0d0
         jloop2: do jcount=1,kn
            kns=kns+1
            j=idpt(kns,1)
            ix=idpt(kns,2); iy=idpt(kns,3); iz=idpt(kns,4)
            xx=rx(i)-rx(j); yy=ry(i)-ry(j); zz=rz(i)-rz(j)
            call calc_replica(xx,yy,zz,rc,ix,iy,iz,istat)
            rjx=rx(i)-xx; rjy=ry(i)-yy; rjz=rz(i)-zz
            rij2=(rjx-rix)**2+(rjy-riy)**2+(rjz-riz)**2
            if (rij2.le.eps) go to 204
            if (rij2.ge.rc2) go to 204
            rij=sqrt(rij2)
            rji=rij; rji2=rij2
            
            ispc=tsp(asp(i)); jspc=tsp(asp(j))
            ii=3; if (ispc==jspc) ii=jspc

            drhoji0drix = xx/rij*drhoa(j,i,0,rji)
            drhoji0driy = yy/rij*drhoa(j,i,0,rji)
            drhoji0driz = zz/rij*drhoa(j,i,0,rji)
            drhoji1drix = xx/rij*drhoa(j,i,1,rji)
            drhoji1driy = yy/rij*drhoa(j,i,1,rji)
            drhoji1driz = zz/rij*drhoa(j,i,1,rji)
            drhoji2drix = xx/rij*drhoa(j,i,2,rji)
            drhoji2driy = yy/rij*drhoa(j,i,2,rji)
            drhoji2driz = zz/rij*drhoa(j,i,2,rji)
            drhoji3drix = xx/rij*drhoa(j,i,3,rji)
            drhoji3driy = yy/rij*drhoa(j,i,3,rji)
            drhoji3driz = zz/rij*drhoa(j,i,3,rji)

            drhoij0drix = xx/rij*drhoa(i,j,0,rij)
            drhoij0driy = yy/rij*drhoa(i,j,0,rij)
            drhoij0driz = zz/rij*drhoa(i,j,0,rij)
            drhoij1drix = xx/rij*drhoa(i,j,1,rij)
            drhoij1driy = yy/rij*drhoa(i,j,1,rij)
            drhoij1driz = zz/rij*drhoa(i,j,1,rij)
            drhoij2drix = xx/rij*drhoa(i,j,2,rij)
            drhoij2driy = yy/rij*drhoa(i,j,2,rij)
            drhoij2driz = zz/rij*drhoa(i,j,2,rij)
            drhoij3drix = xx/rij*drhoa(i,j,3,rij)
            drhoij3driy = yy/rij*drhoa(i,j,3,rij)
            drhoij3driz = zz/rij*drhoa(i,j,3,rij)

            dsjidrix=0.0d0;dsjidriy=0.0d0;dsjidriz=0.0d0

            dttj0drix=0.0d0;dttj1drix=0.0d0;dttj2drix=0.0d0;dttj3drix=0.0d0
            dttj0driy=0.0d0;dttj1driy=0.0d0;dttj2driy=0.0d0;dttj3driy=0.0d0
            dttj0driz=0.0d0;dttj1driz=0.0d0;dttj2driz=0.0d0;dttj3driz=0.0d0
            drhobj0drix=0.0d0;drhobj0driy=0.0d0;drhobj0driz=0.0d0
            drhobj1drix=0.0d0;drhobj1driy=0.0d0;drhobj1driz=0.0d0
            drhobj2drix=0.0d0;drhobj2driy=0.0d0;drhobj2driz=0.0d0
            drhobj3drix=0.0d0;drhobj3driy=0.0d0;drhobj3driz=0.0d0

            knsj=0; if (j>0) then; do jcnt=1,j-1; knsj=knsj+knum(jcnt); enddo; endif
            ixj=ix;iyj=iy;izj=iz
            knsk=kns0

            kloop2: do kcount=1,kn   ! <to calculate S_ij>
               knsk=knsk+1
               k=idpt(knsk,1)
               ix=idpt(knsk,2); iy=idpt(knsk,3); iz=idpt(knsk,4)
               xx=rx(i)-rx(k); yy=ry(i)-ry(k); zz=rz(i)-rz(k)
               call calc_replica(xx,yy,zz,rc,ix,iy,iz,istat)
               rkx=rx(i)-xx; rky=ry(i)-yy; rkz=rz(i)-zz
               rik2=(rkx-rix)**2+(rky-riy)**2+(rkz-riz)**2

               if (rik2.le.eps) go to 304
!               if (rik2.ge.rc2) go to 304
               rik=sqrt(rik2)
               rki=rik; rki2=rik2
               rjk2=(rkx-rjx)**2+(rky-rjy)**2+(rkz-rjz)**2
               if (rjk2.le.eps) go to 304
               rjk=sqrt(rjk2)
               rkj=rjk; rkj2=rjk2

               kspc=tsp(asp(k))

               xxik=(rik2/rij2); xxjk=(rjk2/rij2)
               cos1=(rik2+rij2-rjk2)/(2.0*rik*rij)
               cos2=(rij2+rjk2-rik2)/(2.0*rij*rjk)
               if ((abs(xxik-xxjk)-1.0<eps0).and.(abs(xxik-xxjk)-1.0>-eps0)) then
                  cc=-1.0d0
               else if ((cos1<=0.0).or.(cos2<=0.0)) then
                  cc=-1.0d0
               else
                  cc=( 2.0d0*(xxik+xxjk) - (xxik-xxjk)**2 - 1.0d0 ) / ( 1.0d0-(xxik-xxjk)**2 )
               endif
               if ((tsp(asp(i))==1).and.(tsp(asp(j))==1)) then
                  cmin=ccmin(1); cmax=ccmax(1)
               else if ((tsp(asp(i))==2).and.(tsp(asp(j))==2)) then
                  cmin=ccmin(2); cmax=ccmax(2)
               else
                  cmin=ccmin(3); cmax=ccmax(3)
               endif
               if (cc<=0.0) then
                  sjki=1.0d0
               else if (cc<=cmin) then
                  sjki=0.0d0
               else if (cc<=cmax) then
                  sjki=(1.0d0-((cmax-cc)/(cmax-cmin))**4)**2
               else
                  sjki=1.0d0
               endif

!               f=-rjk2*rji2+rki2*rji2; g=rji2**2-(rjk2-rki2)**2
               f=rji2*(rjk2+rki2-rji2); g=rji2**2-(rjk2-rki2)**2 ! cc=1.0d0+2.0*f/g
!               if (abs(g)>eps) then
               if ((sjki>0.0).and.(sjki<1.0)) then
!               if ((abs(g)>eps).and.(sjki>0.0).and.(sjki<1.0)) then
                  dsjkidc=8.0d0*(1.0d0-( (cmax-cc)/(cmax-cmin) )**4) * (cmax-cc)**3 &
                       / (cmax-cmin)**4
                  dcdrji=( 4.0d0*rij*(rjk2+rki2-2.0d0*rji2)*g-f*8.0d0*rji2*rji )/g**2; dcdrij=dcdrji
                  dcdrki=( 4.0d0*rki*rji2*g-8.0d0*f*(rjk2-rki2)*rki )/g**2; dcdrik=dcdrki
                  dcdrjk=(-4.0d0*rij2*rjk*g+8.0d0*f*(rjk2-rki2)*rjk )/g**2; dcdrkj=dcdrjk ! stress
                  dcdrix=dcdrji*(rix-rjx)/rji+dcdrki*(rix-rkx)/rki
                  dcdriy=dcdrji*(riy-rjy)/rji+dcdrki*(riy-rky)/rki
                  dcdriz=dcdrji*(riz-rjz)/rji+dcdrki*(riz-rkz)/rki
                  dsjkidrix=dsjkidc*dcdrix
                  dsjkidriy=dsjkidc*dcdriy
                  dsjkidriz=dsjkidc*dcdriz
                  dsjkidrij=dsjkidc*dcdrij ! stress
                  dsjkidrjk=dsjkidc*dcdrjk
                  dsjkidrki=dsjkidc*dcdrki
               else
                  dsjkidc=0.0;dsjkidrix=0.0;dsjkidriy=0.0;dsjkidriz=0.0
               endif
!               if (ss(j,i).ne.0.0) then
               if ((sjki>0.0).and.(sjki<1.0)) then
                  dsjidrix=dsjidrix+dsjkidrix*ss(j,i)/sjki
                  dsjidriy=dsjidriy+dsjkidriy*ss(j,i)/sjki
                  dsjidriz=dsjidriz+dsjkidriz*ss(j,i)/sjki
                  do ia=1,3 ! stress
                     do ib=1,3
                        ds_e(j,i,ia,ib)=ds_e(j,i,ia,ib)+ss(j,i)/sjki &
                             *(dsjkidrij*drde(rjx-rix,rjy-riy,rjz-riz,rij,ia,ib) &
                             + dsjkidrjk*drde(rkx-rjx,rky-rjy,rkz-rjz,rjk,ia,ib) &
                             + dsjkidrki*drde(rix-rkx,riy-rky,riz-rkz,rki,ia,ib))
                     enddo
                  enddo
               endif

               xxi=(rij2/rkj2); xxj=(rik2/rkj2)
               cos1=(rik2+rkj2-rij2)/(2.0*rik*rkj)
               cos2=(rij2+rjk2-rik2)/(2.0*rij*rkj)
               if ((abs(xxi-xxj)-1.0<eps0).and.(abs(xxi-xxj)-1.0>-eps0)) then
                  ccs=-1.0d0
               else if ((cos1<=0.0).or.(cos2<=0.0)) then
                  ccs=-1.0d0
               else
                  ccs=(-(xxi-xxj)**2+2.0*(xxi+xxj)-1.0)/(1.0-(xxi-xxj)**2)
               endif
               if ((tsp(asp(k))==1).and.(tsp(asp(j))==1)) then
                  cmin=ccmin(1); cmax=ccmax(1)
               else if ((tsp(asp(k))==2).and.(tsp(asp(j))==2)) then
                  cmin=ccmin(2); cmax=ccmax(2)
               else
                  cmin=ccmin(3); cmax=ccmax(3)
               endif
               if (ccs<=0.0) then
                  skij=1.0d0
               else if (ccs<=cmin) then
                  skij=0.0d0
               else if (ccs<=cmax) then
                  skij=(1.0d0-((cmax-ccs)/(cmax-cmin))**4)**2
               else
                  skij=1.0d0
               endif

!               f=-rki2*rkj2+rji2*rkj2; g=rkj2**2-(rki2-rji2)**2
               f=rkj2*(rki2+rji2-rkj2); g=rkj2**2-(rki2-rji2)**2
!               if (abs(g)>eps) then
               if ((skij>0.0).and.(skij<1.0)) then
                  dskijdcs=8.0d0*(1.0d0-( (cmax-ccs)/(cmax-cmin) )**4) * (cmax-ccs)**3 &
                       / (cmax-cmin)**4
                  dcsdrik=( (4.0d0*rik*rkj2)*g+f*8.0d0*rik*(rik2-rji2) )/g**2
                  dcsdrij=( (4.0d0*rij*rkj2)*g-f*8.0d0*rij*(rik2-rji2) )/g**2
                  dcsdrix=dcsdrik*(rix-rkx)/rik+dcsdrij*(rix-rjx)/rij
                  dcsdriy=dcsdrik*(riy-rky)/rik+dcsdrij*(riy-rjy)/rij
                  dcsdriz=dcsdrik*(riz-rkz)/rik+dcsdrij*(riz-rjz)/rij
                  dskijdrix=dskijdcs*dcsdrix
                  dskijdriy=dskijdcs*dcsdriy
                  dskijdriz=dskijdcs*dcsdriz
               else
                  dskijdcs=0.0;dskijdrix=0.0;dskijdriy=0.0;dskijdriz=0.0
               endif

!               if (skij.ne.0.0) then
               if ((skij>0.0).and.(skij<1.0)) then
                  dskjdrix=ss(k,j)/skij*dskijdrix
                  dskjdriy=ss(k,j)/skij*dskijdriy
                  dskjdriz=ss(k,j)/skij*dskijdriz
               else
                  dskjdrix=0.0d0;dskjdriy=0.0d0;dskjdriz=0.0d0
               endif
!               ds_x(k,j,i)=dskjdrix; ds_y(k,j,i)=dskjdriy; ds_z(k,j,i)=dskjdriz
!               ds_x(j,k,i)=dskjdrix; ds_y(j,k,i)=dskjdriy; ds_z(j,k,i)=dskjdriz
               ds_x(k,j)=dskjdrix; ds_y(k,j)=dskjdriy; ds_z(k,j)=dskjdriz
               ds_x(j,k)=dskjdrix; ds_y(j,k)=dskjdriy; ds_z(j,k)=dskjdriz

               skj=ss(k,j)
              
               if (skj.ne.0.0) then
                  if (skij==0.0) then
                     print *,'ERR maybe cell is too small',i,k,j,rkj,rki,rij
                     stop
                     cycle
                  endif
               dttj0drix=dttj0drix+t00(kspc)*rhoa(k,j,0,rkj)*skj/skij*dskijdrix
               dttj0driy=dttj0driy+t00(kspc)*rhoa(k,j,0,rkj)*skj/skij*dskijdriy
               dttj0driz=dttj0driz+t00(kspc)*rhoa(k,j,0,rkj)*skj/skij*dskijdriz
               dttj1drix=dttj1drix+t01(kspc)*rhoa(k,j,0,rkj)*skj/skij*dskijdrix
               dttj1driy=dttj1driy+t01(kspc)*rhoa(k,j,0,rkj)*skj/skij*dskijdriy
               dttj1driz=dttj1driz+t01(kspc)*rhoa(k,j,0,rkj)*skj/skij*dskijdriz
               dttj2drix=dttj2drix+t02(kspc)*rhoa(k,j,0,rkj)*skj/skij*dskijdrix
               dttj2driy=dttj2driy+t02(kspc)*rhoa(k,j,0,rkj)*skj/skij*dskijdriy
               dttj2driz=dttj2driz+t02(kspc)*rhoa(k,j,0,rkj)*skj/skij*dskijdriz
               dttj3drix=dttj3drix+t03(kspc)*rhoa(k,j,0,rkj)*skj/skij*dskijdrix
               dttj3driy=dttj3driy+t03(kspc)*rhoa(k,j,0,rkj)*skj/skij*dskijdriy
               dttj3driz=dttj3driz+t03(kspc)*rhoa(k,j,0,rkj)*skj/skij*dskijdriz

               drhobj0drix=drhobj0drix+rhoa(k,j,0,rkj)*skj/skij*dskijdrix
               drhobj0driy=drhobj0driy+rhoa(k,j,0,rkj)*skj/skij*dskijdriy
               drhobj0driz=drhobj0driz+rhoa(k,j,0,rkj)*skj/skij*dskijdriz


               xjk(1)=(rkx-rjx)/rjk; xjk(2)=(rky-rjy)/rjk; xjk(3)=(rkz-rjz)/rjk
               if (rhobp(j,2).ne.0.0) then
                  drhobj2drix=drhobj2drix &
                       -rs(j)*( rhoa(k,j,2,rkj)*skj/skij*dskijdrix )/rhobp(j,2)/3.0d0  ! <== corrected
                  drhobj2driy=drhobj2driy &
                       -rs(j)*( rhoa(k,j,2,rkj)*skj/skij*dskijdriy )/rhobp(j,2)/3.0d0
                  drhobj2driz=drhobj2driz &
                       -rs(j)*( rhoa(k,j,2,rkj)*skj/skij*dskijdriz )/rhobp(j,2)/3.0d0
               endif
               do ia=1,3
                  if (rhobp(j,1).ne.0.0) then
                  drhobj1drix=drhobj1drix+xrs(j,ia)*( xjk(ia)*rhoa(k,j,1,rkj)*skj/skij*dskijdrix )/rhobp(j,1)
                  drhobj1driy=drhobj1driy+xrs(j,ia)*( xjk(ia)*rhoa(k,j,1,rkj)*skj/skij*dskijdriy )/rhobp(j,1)
                  drhobj1driz=drhobj1driz+xrs(j,ia)*( xjk(ia)*rhoa(k,j,1,rkj)*skj/skij*dskijdriz )/rhobp(j,1) 
                  endif
                  do ip=1,3
                     if (rhobp(j,2).ne.0.0) then
                     drhobj2drix=drhobj2drix+xxrs(j,ia,ip)*( &
                          xjk(ia)*xjk(ip)*rhoa(k,j,2,rkj)*skj/skij*dskijdrix )/rhobp(j,2)
                     drhobj2driy=drhobj2driy+xxrs(j,ia,ip)*( &
                          xjk(ia)*xjk(ip)*rhoa(k,j,2,rkj)*skj/skij*dskijdriy )/rhobp(j,2)
                     drhobj2driz=drhobj2driz+xxrs(j,ia,ip)*( &
                          xjk(ia)*xjk(ip)*rhoa(k,j,2,rkj)*skj/skij*dskijdriz )/rhobp(j,2)
                     endif
                     do iq=1,3
                        if (rhobp(j,3).ne.0.0) then
                        drhobj3drix=drhobj3drix+xxxrs(j,ia,ip,iq)*( &
                             xjk(ia)*xjk(ip)*xjk(iq)*rhoa(k,j,3,rkj)*skj/skij*dskijdrix )/rhobp(j,3)  ! <== corrected
                        drhobj3driy=drhobj3driy+xxxrs(j,ia,ip,iq)*( &
                             xjk(ia)*xjk(ip)*xjk(iq)*rhoa(k,j,3,rkj)*skj/skij*dskijdriy )/rhobp(j,3)
                        drhobj3driz=drhobj3driz+xxxrs(j,ia,ip,iq)*( &
                             xjk(ia)*xjk(ip)*xjk(iq)*rhoa(k,j,3,rkj)*skj/skij*dskijdriz )/rhobp(j,3)
                        endif
                     enddo
                  enddo
               enddo
               endif
            
304            continue
            end do kloop2

!            ds_x(j,i,i)=dsjidrix; ds_y(j,i,i)=dsjidriy; ds_z(j,i,i)=dsjidriz
            ds_x(j,i)=dsjidrix; ds_y(j,i)=dsjidriy; ds_z(j,i)=dsjidriz
            ds_x(i,j)=dsjidrix; ds_y(i,j)=dsjidriy; ds_z(i,j)=dsjidriz


            drhoi0dti0=rhos(i)*(-0.5d0*ss0/zz0**2)
            drhoi0dti1=rhos(i)*(-0.5d0*ss1/zz0**2)
            drhoi0dti2=rhos(i)*(-0.5d0*ss2/zz0**2)
            drhoi0dti3=rhos(i)*(-0.5d0*ss3/zz0**2)

            drhobi0drix=drhobi0drix+drhoji0drix*ss(j,i)+rhoa(j,i,0,rji)*dsjidrix
            drhobi0driy=drhobi0driy+drhoji0driy*ss(j,i)+rhoa(j,i,0,rji)*dsjidriy
            drhobi0driz=drhobi0driz+drhoji0driz*ss(j,i)+rhoa(j,i,0,rji)*dsjidriz

            drhobj0drix=drhobj0drix+drhoij0drix*ss(i,j)+rhoa(i,j,0,rij)*dsjidrix
            drhobj0driy=drhobj0driy+drhoij0driy*ss(i,j)+rhoa(i,j,0,rij)*dsjidriy
            drhobj0driz=drhobj0driz+drhoij0driz*ss(i,j)+rhoa(i,j,0,rij)*dsjidriz

            xij(1)=(rjx-rix)/rji; xij(2)=(rjy-riy)/rji; xij(3)=(rjz-riz)/rji
            dxijdrix(1)=-1.0d0/rji+xij(1)/rji*xij(1)
            dxijdriy(1)=           xij(1)/rji*xij(2)
            dxijdriz(1)=           xij(1)/rji*xij(3)
            dxijdrix(2)=           xij(2)/rji*xij(1)
            dxijdriy(2)=-1.0d0/rji+xij(2)/rji*xij(2)
            dxijdriz(2)=           xij(2)/rji*xij(3)
            dxijdrix(3)=           xij(3)/rji*xij(1)
            dxijdriy(3)=           xij(3)/rji*xij(2)
            dxijdriz(3)=-1.0d0/rji+xij(3)/rji*xij(3)

            sji=ss(j,i)  ! sji <== s(j,i)
            if (sji.ne.0.0) then
               if (rhobp(i,2).ne.0.0) then
                  drhobi2drix=drhobi2drix &
                       -rs(i)*(drhoji2drix*sji+rhoa(j,i,2,rji)*dsjidrix)/rhobp(i,2)/3.0d0
                  drhobi2driy=drhobi2driy &
                       -rs(i)*(drhoji2driy*sji+rhoa(j,i,2,rji)*dsjidriy)/rhobp(i,2)/3.0d0
                  drhobi2driz=drhobi2driz &
                       -rs(i)*(drhoji2driz*sji+rhoa(j,i,2,rji)*dsjidriz)/rhobp(i,2)/3.0d0
! stress
               do iaa=1,3; do ibb=1,3
                  dr2de=drhoa(j,i,2,rij)*drde(rjx-rix,rjy-riy,rjz-riz,rij,iaa,ibb)*sji
                  drhobi2de(iaa,ibb)=drhobi2de(iaa,ibb) &
                       -rs(i)*(dr2de*sji+rhoa(j,i,2,rji)*ds_e(j,i,iaa,ibb))/rhobp(i,2)/3.0d0
               enddo; enddo
               endif
            do ia=1,3
               if (rhobp(i,1).ne.0.0) then
               drhobi1drix=drhobi1drix+xrs(i,ia)*( dxijdrix(ia)*rhoa(j,i,1,rji)*sji &
                    +xij(ia)*drhoji1drix*sji+xij(ia)*rhoa(j,i,1,rji)*dsjidrix )/rhobp(i,1)
               drhobi1driy=drhobi1driy+xrs(i,ia)*( dxijdriy(ia)*rhoa(j,i,1,rji)*sji &
                    +xij(ia)*drhoji1driy*sji+xij(ia)*rhoa(j,i,1,rji)*dsjidriy )/rhobp(i,1)
               drhobi1driz=drhobi1driz+xrs(i,ia)*( dxijdriz(ia)*rhoa(j,i,1,rji)*sji &
                    +xij(ia)*drhoji1driz*sji+xij(ia)*rhoa(j,i,1,rji)*dsjidriz )/rhobp(i,1)
! stress
               do iaa=1,3; do ibb=1,3
                  dxde1=dxde(rix,riy,riz,rjx,rjy,rjz,rij,ia,iaa,ibb)
                  dr1de=drhoa(j,i,1,rij)*drde(rjx-rix,rjy-riy,rjz-riz,rij,iaa,ibb)*sji
                  drhobi1de(iaa,ibb)=drhobi1de(iaa,ibb)+xrs(i,ia)*( dxde1*rhoa(j,i,1,rji)*sji &
                       +xij(ia)*dr1de*sji+xij(ia)*rhoa(j,i,1,rji)*ds_e(j,i,iaa,ibb) )/rhobp(i,1)
               enddo; enddo
               endif
               do ip=1,3
                  if (rhobp(i,2).ne.0.0) then
                  drhobi2drix=drhobi2drix+xxrs(i,ia,ip)*( dxijdrix(ia)*xij(ip)*rhoa(j,i,2,rji)*sji &
                       +xij(ia)*dxijdrix(ip)*rhoa(j,i,2,rji)*sji &
                       +xij(ia)*xij(ip)*drhoji2drix*sji &
                       +xij(ia)*xij(ip)*rhoa(j,i,2,rji)*dsjidrix )/rhobp(i,2)
                  drhobi2driy=drhobi2driy+xxrs(i,ia,ip)*( dxijdriy(ia)*xij(ip)*rhoa(j,i,2,rji)*sji &
                       +xij(ia)*dxijdriy(ip)*rhoa(j,i,2,rji)*sji &
                       +xij(ia)*xij(ip)*drhoji2driy*sji &
                       +xij(ia)*xij(ip)*rhoa(j,i,2,rji)*dsjidriy )/rhobp(i,2)
                  drhobi2driz=drhobi2driz+xxrs(i,ia,ip)*( dxijdriz(ia)*xij(ip)*rhoa(j,i,2,rji)*sji &
                       +xij(ia)*dxijdriz(ip)*rhoa(j,i,2,rji)*sji &
                       +xij(ia)*xij(ip)*drhoji2driz*sji &
                       +xij(ia)*xij(ip)*rhoa(j,i,2,rji)*dsjidriz )/rhobp(i,2)
! stress
                  do iaa=1,3; do ibb=1,3
                     dxde1=dxde(rix,riy,riz,rjx,rjy,rjz,rij,ia,iaa,ibb)
                     dxde2=dxde(rix,riy,riz,rjx,rjy,rjz,rij,ip,iaa,ibb)
                     dr2de=drhoa(j,i,2,rij)*drde(rjx-rix,rjy-riy,rjz-riz,rij,iaa,ibb)*sji
                     drhobi2de(iaa,ibb)=drhobi2de(iaa,ibb)+xxrs(i,ia,ip)*( &
                          dxde1*xij(ip)*rhoa(j,i,2,rji)*sji &
                          +xij(ia)*dxde2*rhoa(j,i,2,rji)*sji &
                          +xij(ia)*xij(ip)*dr2de*sji &
                          +xij(ia)*xij(ip)*rhoa(j,i,2,rji)*ds_e(j,i,iaa,ibb) )/rhobp(i,2)
                  enddo; enddo
                  endif
                  do iq=1,3
                     if (rhobp(i,3).ne.0.0) then
!                     drhobi3drix=drhobi3drix &
!                          +xxxrs(i,ia,ip,iq)*xij(ia)*xij(ip)*xij(iq)*rhoa(j,i,3,rji)*sji &
!                          *( dxijdrix(ia)/xij(ia)+dxijdrix(ip)/xij(ip)+dxijdrix(iq)/xij(iq) &
!                          +drhoji3drix/rhoa(j,i,3,rji)+dsjidrix/sji )/rhobp(i,3)
!                     drhobi3driy=drhobi3driy &
!                          +xxxrs(i,ia,ip,iq)*xij(ia)*xij(ip)*xij(iq)*rhoa(j,i,3,rji)*sji &
!                          *( dxijdriy(ia)/xij(ia)+dxijdriy(ip)/xij(ip)+dxijdriy(iq)/xij(iq) &
!                          +drhoji3driy/rhoa(j,i,3,rji)+dsjidriy/sji )/rhobp(i,3)
!                     drhobi3driz=drhobi3driz &
!                          +xxxrs(i,ia,ip,iq)*xij(ia)*xij(ip)*xij(iq)*rhoa(j,i,3,rji)*sji &
!                          *( dxijdriz(ia)/xij(ia)+dxijdriz(ip)/xij(ip)+dxijdriz(iq)/xij(iq) &
!                          +drhoji3driz/rhoa(j,i,3,rji)+dsjidriz/sji )/rhobp(i,3)
                     drhobi3drix=drhobi3drix+xxxrs(i,ia,ip,iq) &
                          *(dxijdrix(ia)*xij(ip)*xij(iq)*rhoa(j,i,3,rji)*sji &
                          + xij(ia)*dxijdrix(ip)*xij(iq)*rhoa(j,i,3,rji)*sji &
                          + xij(ia)*xij(ip)*dxijdrix(iq)*rhoa(j,i,3,rji)*sji &
                          + xij(ia)*xij(ip)*xij(iq)*drhoji3drix*sji &
                          + xij(ia)*xij(ip)*xij(iq)*rhoa(j,i,3,rji)*dsjidrix )/rhobp(i,3)
                     drhobi3driy=drhobi3driy+xxxrs(i,ia,ip,iq) &
                          *(dxijdriy(ia)*xij(ip)*xij(iq)*rhoa(j,i,3,rji)*sji &
                          + xij(ia)*dxijdriy(ip)*xij(iq)*rhoa(j,i,3,rji)*sji &
                          + xij(ia)*xij(ip)*dxijdriy(iq)*rhoa(j,i,3,rji)*sji &
                          + xij(ia)*xij(ip)*xij(iq)*drhoji3driy*sji &
                          + xij(ia)*xij(ip)*xij(iq)*rhoa(j,i,3,rji)*dsjidriy )/rhobp(i,3)
                     drhobi3driz=drhobi3driz+xxxrs(i,ia,ip,iq) &
                          *(dxijdriz(ia)*xij(ip)*xij(iq)*rhoa(j,i,3,rji)*sji &
                          + xij(ia)*dxijdriz(ip)*xij(iq)*rhoa(j,i,3,rji)*sji &
                          + xij(ia)*xij(ip)*dxijdriz(iq)*rhoa(j,i,3,rji)*sji &
                          + xij(ia)*xij(ip)*xij(iq)*drhoji3driz*sji &
                          + xij(ia)*xij(ip)*xij(iq)*rhoa(j,i,3,rji)*dsjidriz )/rhobp(i,3)

! stress
                     do iaa=1,3; do ibb=1,3
                        dxde1=dxde(rix,riy,riz,rjx,rjy,rjz,rij,ia,iaa,ibb)
                        dxde2=dxde(rix,riy,riz,rjx,rjy,rjz,rij,ip,iaa,ibb)
                        dxde3=dxde(rix,riy,riz,rjx,rjy,rjz,rij,iq,iaa,ibb)
                        dr3de=drhoa(j,i,3,rij)*drde(rjx-rix,rjy-riy,rjz-riz,rij,iaa,ibb)*sji
!                        drhobi3de(iaa,ibb)=drhobi3de(iaa,ibb) &
!                          +xxxrs(i,ia,ip,iq)*xij(ia)*xij(ip)*xij(iq)*rhoa(j,i,3,rji)*sji &
!                          *( dxde1/xij(ia)+dxde2/xij(ip)+dxde3/xij(iq) &
!                          +dr3de/rhoa(j,i,3,rji)+ds_e(j,i,iaa,ibb)/sji )/rhobp(i,3)
                        drhobi3de(iaa,ibb)=drhobi3de(iaa,ibb)+xxxrs(i,ia,ip,iq) &
                             *(dxde1*xij(ip)*xij(iq)*rhoa(j,i,3,rji)*sji &
                             + xij(ia)*dxde2*xij(iq)*rhoa(j,i,3,rji)*sji &
                             + xij(ia)*xij(ip)*dxde3*rhoa(j,i,3,rji)*sji &
                             + xij(ia)*xij(ip)*xij(iq)*dr3de*sji &
                             + xij(ia)*xij(ip)*xij(iq)*rhoa(j,i,3,rji)*ds_e(j,i,iaa,ibb) )/rhobp(i,3)
                     enddo; enddo
                     endif
                  enddo
               enddo
            enddo
            endif

            xji(1)=(rix-rjx)/rji; xji(2)=(riy-rjy)/rji; xji(3)=(riz-rjz)/rji
            dxjidrix(1)= 1.0d0/rji-xji(1)/rji*xji(1)
            dxjidriy(1)=          -xji(1)/rji*xji(2)
            dxjidriz(1)=          -xji(1)/rji*xji(3)
            dxjidrix(2)=          -xji(2)/rji*xji(1)
            dxjidriy(2)= 1.0d0/rji-xji(2)/rji*xji(2)
            dxjidriz(2)=          -xji(2)/rji*xji(3)
            dxjidrix(3)=          -xji(3)/rji*xji(1)
            dxjidriy(3)=          -xji(3)/rji*xji(2)
            dxjidriz(3)= 1.0d0/rji-xji(3)/rji*xji(3)
            if (sji.ne.0.0) then
               if (rhobp(j,2).ne.0.0) then
                  drhobj2drix=drhobj2drix &
                       -rs(j)*(drhoij2drix*sji+rhoa(i,j,2,rij)*dsjidrix)/rhobp(j,2)/3.0d0
                  drhobj2driy=drhobj2driy &
                       -rs(j)*(drhoij2driy*sji+rhoa(i,j,2,rij)*dsjidriy)/rhobp(j,2)/3.0d0
                  drhobj2driz=drhobj2driz &
                       -rs(j)*(drhoij2driz*sji+rhoa(i,j,2,rij)*dsjidriz)/rhobp(j,2)/3.0d0
               endif
            do ia=1,3
               if (rhobp(j,1).ne.0.0) then
               drhobj1drix=drhobj1drix+xrs(j,ia)*( dxjidrix(ia)*rhoa(i,j,1,rij)*sji &
                    +xji(ia)*drhoij1drix*sji+xji(ia)*rhoa(i,j,1,rij)*dsjidrix )/rhobp(j,1)
               drhobj1driy=drhobj1driy+xrs(j,ia)*( dxjidriy(ia)*rhoa(i,j,1,rij)*sji &
                    +xji(ia)*drhoij1driy*sji+xji(ia)*rhoa(i,j,1,rij)*dsjidriy )/rhobp(j,1)
               drhobj1driz=drhobj1driz+xrs(j,ia)*( dxjidriz(ia)*rhoa(i,j,1,rij)*sji &
                    +xji(ia)*drhoij1driz*sji+xji(ia)*rhoa(i,j,1,rij)*dsjidriz )/rhobp(j,1)
               endif
               do ip=1,3
                  if (rhobp(j,2).ne.0.0) then
                  drhobj2drix=drhobj2drix+xxrs(j,ia,ip)*( dxjidrix(ia)*xji(ip)*rhoa(i,j,2,rij)*sji &
                       +xji(ia)*dxjidrix(ip)*rhoa(i,j,2,rij)*sji &
                       +xji(ia)*xji(ip)*drhoij2drix*sji &
                       +xji(ia)*xji(ip)*rhoa(i,j,2,rij)*dsjidrix )/rhobp(j,2)
                  drhobj2driy=drhobj2driy+xxrs(j,ia,ip)*( dxjidriy(ia)*xji(ip)*rhoa(i,j,2,rij)*sji &
                       +xji(ia)*dxjidriy(ip)*rhoa(i,j,2,rij)*sji &
                       +xji(ia)*xji(ip)*drhoij2driy*sji &
                       +xji(ia)*xji(ip)*rhoa(i,j,2,rij)*dsjidrix )/rhobp(j,2)
                  drhobj2driz=drhobj2driz+xxrs(j,ia,ip)*( dxjidriz(ia)*xji(ip)*rhoa(i,j,2,rij)*sji &
                       +xji(ia)*dxjidriz(ip)*rhoa(i,j,2,rij)*sji &
                       +xji(ia)*xji(ip)*drhoij2driz*sji &
                       +xji(ia)*xji(ip)*rhoa(i,j,2,rij)*dsjidrix )/rhobp(j,2)
                  endif
                  do iq=1,3
                     if (rhobp(j,3).ne.0.0) then
!                     drhobj3drix=drhobj3drix &
!                          +xxxrs(j,ia,ip,iq)*xji(ia)*xji(ip)*xji(iq)*rhoa(i,j,3,rij)*sji &
!                          *( dxjidrix(ia)/xji(ia)+dxjidrix(ip)/xji(ip)+dxjidrix(iq)/xji(iq) &
!                          +drhoij3drix/rhoa(i,j,3,rij)+dsjidrix/sji )/rhobp(j,3)
!                     drhobj3driy=drhobj3driy &
!                          +xxxrs(j,ia,ip,iq)*xji(ia)*xji(ip)*xji(iq)*rhoa(i,j,3,rij)*sji &
!                          *( dxjidriy(ia)/xji(ia)+dxjidriy(ip)/xji(ip)+dxjidriy(iq)/xji(iq) &
!                          +drhoij3driy/rhoa(i,j,3,rij)+dsjidriy/sji )/rhobp(j,3)
!                     drhobj3driz=drhobj3driz &
!                          +xxxrs(j,ia,ip,iq)*xji(ia)*xji(ip)*xji(iq)*rhoa(i,j,3,rij)*sji &
!                          *( dxjidriz(ia)/xji(ia)+dxjidriz(ip)/xji(ip)+dxjidriz(iq)/xji(iq) &
!                          +drhoij3driz/rhoa(i,j,3,rij)+dsjidriz/sji )/rhobp(j,3)
                     drhobj3drix=drhobj3drix+xxxrs(j,ia,ip,iq) &
                          *(dxjidrix(ia)*xji(ip)*xji(iq)*rhoa(i,j,3,rij)*sji &
                          + xji(ia)*dxjidrix(ip)*xji(iq)*rhoa(i,j,3,rij)*sji &
                          + xji(ia)*xji(ip)*dxjidrix(iq)*rhoa(i,j,3,rij)*sji &
                          + xji(ia)*xji(ip)*xji(iq)*drhoij3drix*sji &
                          + xji(ia)*xji(ip)*xji(iq)*rhoa(i,j,3,rij)*dsjidrix )/rhobp(j,3)
                     drhobj3driy=drhobj3driy+xxxrs(j,ia,ip,iq) &
                          *(dxjidriy(ia)*xji(ip)*xji(iq)*rhoa(i,j,3,rij)*sji &
                          + xji(ia)*dxjidriy(ip)*xji(iq)*rhoa(i,j,3,rij)*sji &
                          + xji(ia)*xji(ip)*dxjidriy(iq)*rhoa(i,j,3,rij)*sji &
                          + xji(ia)*xji(ip)*xji(iq)*drhoij3driy*sji &
                          + xji(ia)*xji(ip)*xji(iq)*rhoa(i,j,3,rij)*dsjidriy )/rhobp(j,3)
                     drhobj3driz=drhobj3driz+xxxrs(j,ia,ip,iq) &
                          *(dxjidriz(ia)*xji(ip)*xji(iq)*rhoa(i,j,3,rij)*sji &
                          + xji(ia)*dxjidriz(ip)*xji(iq)*rhoa(i,j,3,rij)*sji &
                          + xji(ia)*xji(ip)*dxjidriz(iq)*rhoa(i,j,3,rij)*sji &
                          + xji(ia)*xji(ip)*xji(iq)*drhoij3driz*sji &
                          + xji(ia)*xji(ip)*xji(iq)*rhoa(i,j,3,rij)*dsjidriz )/rhobp(j,3)
                  endif
                  enddo
               enddo
            enddo
            endif

            dtti0drix=dtti0drix+t00(jspc)*(drhoji0drix*sji +rhoa(j,i,0,rij)*dsjidrix)
            dtti0driy=dtti0driy+t00(jspc)*(drhoji0driy*sji +rhoa(j,i,0,rij)*dsjidriy)
            dtti0driz=dtti0driz+t00(jspc)*(drhoji0driz*sji +rhoa(j,i,0,rij)*dsjidriz)
            dtti1drix=dtti1drix+t01(jspc)*(drhoji0drix*sji +rhoa(j,i,0,rij)*dsjidrix)
            dtti1driy=dtti1driy+t01(jspc)*(drhoji0driy*sji +rhoa(j,i,0,rij)*dsjidriy)
            dtti1driz=dtti1driz+t01(jspc)*(drhoji0driz*sji +rhoa(j,i,0,rij)*dsjidriz)
            dtti2drix=dtti2drix+t02(jspc)*(drhoji0drix*sji +rhoa(j,i,0,rij)*dsjidrix)
            dtti2driy=dtti2driy+t02(jspc)*(drhoji0driy*sji +rhoa(j,i,0,rij)*dsjidriy)
            dtti2driz=dtti2driz+t02(jspc)*(drhoji0driz*sji +rhoa(j,i,0,rij)*dsjidriz)
            dtti3drix=dtti3drix+t03(jspc)*(drhoji0drix*sji +rhoa(j,i,0,rij)*dsjidrix)
            dtti3driy=dtti3driy+t03(jspc)*(drhoji0driy*sji +rhoa(j,i,0,rij)*dsjidriy)
            dtti3driz=dtti3driz+t03(jspc)*(drhoji0driz*sji +rhoa(j,i,0,rij)*dsjidriz)
! stress
            do ia=1,3; do ib=1,3
               drsde=drhoa(j,i,0,rij)*drde(rjx-rix,rjy-riy,rjz-riz,rij,ia,ib)*sji &
                    +rhoa(j,i,0,rij)*ds_e(j,i,ia,ib)
               drhobi0de(ia,ib)=drhobi0de(ia,ib)+drsde
               dttide(0,ia,ib)=dttide(0,ia,ib)+t00(jspc)*drsde
               dttide(1,ia,ib)=dttide(1,ia,ib)+t01(jspc)*drsde
               dttide(2,ia,ib)=dttide(2,ia,ib)+t02(jspc)*drsde
               dttide(3,ia,ib)=dttide(3,ia,ib)+t03(jspc)*drsde
            enddo; enddo

            drhobjdrho0j=-rhob(j)/rhos(j)
            drhobjdrhob0j=rhob(j)/rhobp(j,0)**3 &
                 *(rhobp(j,1)**2*t(j,1)**2+rhobp(j,2)**2*t(j,2)**2+rhobp(j,3)**2*t(j,3)**2) &
                 +rhob(j)/rhobp(j,0)
            drhobjdrhob1j=-rhob(j)/rhobp(j,0)**2 * rhobp(j,1)*t(j,1)**2
            drhobjdrhob2j=-rhob(j)/rhobp(j,0)**2 * rhobp(j,2)*t(j,2)**2
            drhobjdrhob3j=-rhob(j)/rhobp(j,0)**2 * rhobp(j,3)*t(j,3)**2
            drhobjdtj1=-rhob(j)/rhobp(j,0)**2 * rhobp(j,1)**2*t(j,1)
            drhobjdtj2=-rhob(j)/rhobp(j,0)**2 * rhobp(j,2)**2*t(j,2)
            drhobjdtj3=-rhob(j)/rhobp(j,0)**2 * rhobp(j,3)**2*t(j,3)

            dttj0drix=dttj0drix+t00(ispc)*(drhoij0drix*sji +rhoa(i,j,0,rij)*dsjidrix)
            dttj0driy=dttj0driy+t00(ispc)*(drhoij0driy*sji +rhoa(i,j,0,rij)*dsjidriy)
            dttj0driz=dttj0driz+t00(ispc)*(drhoij0driz*sji +rhoa(i,j,0,rij)*dsjidriz)
            dttj1drix=dttj1drix+t01(ispc)*(drhoij0drix*sji +rhoa(i,j,0,rij)*dsjidrix)
            dttj1driy=dttj1driy+t01(ispc)*(drhoij0driy*sji +rhoa(i,j,0,rij)*dsjidriy)
            dttj1driz=dttj1driz+t01(ispc)*(drhoij0driz*sji +rhoa(i,j,0,rij)*dsjidriz)
            dttj2drix=dttj2drix+t02(ispc)*(drhoij0drix*sji +rhoa(i,j,0,rij)*dsjidrix)
            dttj2driy=dttj2driy+t02(ispc)*(drhoij0driy*sji +rhoa(i,j,0,rij)*dsjidriy)
            dttj2driz=dttj2driz+t02(ispc)*(drhoij0driz*sji +rhoa(i,j,0,rij)*dsjidriz)
            dttj3drix=dttj3drix+t03(ispc)*(drhoij0drix*sji +rhoa(i,j,0,rij)*dsjidrix)
            dttj3driy=dttj3driy+t03(ispc)*(drhoij0driy*sji +rhoa(i,j,0,rij)*dsjidriy)
            dttj3driz=dttj3driz+t03(ispc)*(drhoij0driz*sji +rhoa(i,j,0,rij)*dsjidriz)

            dtj0drix=dttj0drix/rhobp(j,0)-ttb(j,0)/rhobp(j,0)**2*drhobj0drix
            dtj0driy=dttj0driy/rhobp(j,0)-ttb(j,0)/rhobp(j,0)**2*drhobj0driy
            dtj0driz=dttj0driz/rhobp(j,0)-ttb(j,0)/rhobp(j,0)**2*drhobj0driz
            dtj1drix=dttj1drix/rhobp(j,0)-ttb(j,1)/rhobp(j,0)**2*drhobj0drix
            dtj1driy=dttj1driy/rhobp(j,0)-ttb(j,1)/rhobp(j,0)**2*drhobj0driy
            dtj1driz=dttj1driz/rhobp(j,0)-ttb(j,1)/rhobp(j,0)**2*drhobj0driz
            dtj2drix=dttj2drix/rhobp(j,0)-ttb(j,2)/rhobp(j,0)**2*drhobj0drix
            dtj2driy=dttj2driy/rhobp(j,0)-ttb(j,2)/rhobp(j,0)**2*drhobj0driy
            dtj2driz=dttj2driz/rhobp(j,0)-ttb(j,2)/rhobp(j,0)**2*drhobj0driz
            dtj3drix=dttj3drix/rhobp(j,0)-ttb(j,3)/rhobp(j,0)**2*drhobj0drix
            dtj3driy=dttj3driy/rhobp(j,0)-ttb(j,3)/rhobp(j,0)**2*drhobj0driy
            dtj3driz=dttj3driz/rhobp(j,0)-ttb(j,3)/rhobp(j,0)**2*drhobj0driz

            drhoj0dtj0=rhos(j)*(-0.5d0*ss0/zz0**2)
            drhoj0dtj1=rhos(j)*(-0.5d0*ss1/zz0**2)
            drhoj0dtj2=rhos(j)*(-0.5d0*ss2/zz0**2)
            drhoj0dtj3=rhos(j)*(-0.5d0*ss3/zz0**2)

            drhoj0drix=drhoj0dtj1*dtj1drix+drhoj0dtj2*dtj2drix+drhoj0dtj3*dtj3drix  ! dti3 -> dtj3 08/12
            drhoj0driy=drhoj0dtj1*dtj1driy+drhoj0dtj2*dtj2driy+drhoj0dtj3*dtj3driy
            drhoj0driz=drhoj0dtj1*dtj1driz+drhoj0dtj2*dtj2driz+drhoj0dtj3*dtj3driz

            drhobjdrho0j=-rhob(j)/rhos(j)
            drhobjdrhob0j=rhob(j)/rhobp(j,0)**3 &
                 *(rhobp(j,1)**2*t(j,1)**2+rhobp(j,2)**2*t(j,2)**2+rhobp(j,3)**2*t(j,3)**2) &
                 +rhob(j)/rhobp(j,0)
            drhobjdrhob1j=-rhob(j)/rhobp(j,0)**2 * rhobp(j,1)*t(j,1)**2
            drhobjdrhob2j=-rhob(j)/rhobp(j,0)**2 * rhobp(j,2)*t(j,2)**2
            drhobjdrhob3j=-rhob(j)/rhobp(j,0)**2 * rhobp(j,3)*t(j,3)**2
            drhobjdtj1=-rhob(j)/rhobp(j,0)**2 * rhobp(j,1)**2*t(j,1)
            drhobjdtj2=-rhob(j)/rhobp(j,0)**2 * rhobp(j,2)**2*t(j,2)
            drhobjdtj3=-rhob(j)/rhobp(j,0)**2 * rhobp(j,3)**2*t(j,3)

            drhobjdrix=drhobjdrho0j*drhoj0drix &
                 +drhobjdrhob0j*drhobj0drix+drhobjdrhob1j*drhobj1drix &
                 +drhobjdrhob2j*drhobj2drix+drhobjdrhob3j*drhobj3drix &
                 +drhobjdtj1*dtj1drix+drhobjdtj2*dtj2drix+drhobjdtj3*dtj3drix
            drhobjdriy=drhobjdrho0j*drhoj0driy &
                 +drhobjdrhob0j*drhobj0driy+drhobjdrhob1j*drhobj1driy &
                 +drhobjdrhob2j*drhobj2driy+drhobjdrhob3j*drhobj3driy &
                 +drhobjdtj1*dtj1driy+drhobjdtj2*dtj2driy+drhobjdtj3*dtj3driy
            drhobjdriz=drhobjdrho0j*drhoj0driz &
                 +drhobjdrhob0j*drhobj0driz+drhobjdrhob1j*drhobj1driz &
                 +drhobjdrhob2j*drhobj2driz+drhobjdrhob3j*drhobj3driz &
                 +drhobjdtj1*dtj1driz+drhobjdtj2*dtj2driz+drhobjdtj3*dtj3driz

            drhob_x(j,i)=drhobjdrix; drhob_y(j,i)=drhobjdriy; drhob_z(j,i)=drhobjdriz

!            if ((i==1).and.(j==5)) then
!               debug_val(1)=xij(1)
!               debug_val(2)=dxde(rix,riy,riz,rjx,rjy,rjz,rij,1,1,1)
!           endif

!            if (i==1) print *,j
!            if ((i==1).and.(j==5)) then
!               debug_val(2)=drhobjdriy
!               debug_val(2)=drhobj2driy
!            endif
204         continue
         end do jloop2

         dti0drix=dtti0drix/rhobp(i,0)-ttb(i,0)/rhobp(i,0)**2*drhobi0drix
         dti0driy=dtti0driy/rhobp(i,0)-ttb(i,0)/rhobp(i,0)**2*drhobi0driy
         dti0driz=dtti0driz/rhobp(i,0)-ttb(i,0)/rhobp(i,0)**2*drhobi0driz
         dti1drix=dtti1drix/rhobp(i,0)-ttb(i,1)/rhobp(i,0)**2*drhobi0drix
         dti1driy=dtti1driy/rhobp(i,0)-ttb(i,1)/rhobp(i,0)**2*drhobi0driy
         dti1driz=dtti1driz/rhobp(i,0)-ttb(i,1)/rhobp(i,0)**2*drhobi0driz
         dti2drix=dtti2drix/rhobp(i,0)-ttb(i,2)/rhobp(i,0)**2*drhobi0drix
         dti2driy=dtti2driy/rhobp(i,0)-ttb(i,2)/rhobp(i,0)**2*drhobi0driy
         dti2driz=dtti2driz/rhobp(i,0)-ttb(i,2)/rhobp(i,0)**2*drhobi0driz
         dti3drix=dtti3drix/rhobp(i,0)-ttb(i,3)/rhobp(i,0)**2*drhobi0drix
         dti3driy=dtti3driy/rhobp(i,0)-ttb(i,3)/rhobp(i,0)**2*drhobi0driy
         dti3driz=dtti3driz/rhobp(i,0)-ttb(i,3)/rhobp(i,0)**2*drhobi0driz

! stress
         do ii=0,3; do ia=1,3; do ib=1,3
            dtide(ii,ia,ib)=dttide(ii,ia,ib)/rhobp(i,0)-ttb(i,ii)/rhobp(i,0)**2*drhobi0de(ia,ib)
         enddo; enddo; enddo

         drhoi0drix=drhoi0dti1*dti1drix+drhoi0dti2*dti2drix+drhoi0dti3*dti3drix
         drhoi0driy=drhoi0dti1*dti1driy+drhoi0dti2*dti2driy+drhoi0dti3*dti3driy
         drhoi0driz=drhoi0dti1*dti1driz+drhoi0dti2*dti2driz+drhoi0dti3*dti3driz

! stress
         do ia=1,3; do ib=1,3
            drhoi0de(ia,ib)=drhoi0dti1*dtide(1,ia,ib)+drhoi0dti2*dtide(2,ia,ib)+drhoi0dti3*dtide(3,ia,ib)
         enddo; enddo

         drhobidrho0i=-rhob(i)/rhos(i)
         drhobidrhob0i=rhob(i)/rhobp(i,0)**3 &
              *(rhobp(i,1)**2*t(i,1)**2+rhobp(i,2)**2*t(i,2)**2+rhobp(i,3)**2*t(i,3)**2) &
              +rhob(i)/rhobp(i,0)
         drhobidrhob1i=-rhob(i)/rhobp(i,0)**2 * rhobp(i,1)*t(i,1)**2
         drhobidrhob2i=-rhob(i)/rhobp(i,0)**2 * rhobp(i,2)*t(i,2)**2
         drhobidrhob3i=-rhob(i)/rhobp(i,0)**2 * rhobp(i,3)*t(i,3)**2
         drhobidti1=-rhob(i)/rhobp(i,0)**2 * rhobp(i,1)**2*t(i,1)
         drhobidti2=-rhob(i)/rhobp(i,0)**2 * rhobp(i,2)**2*t(i,2)
         drhobidti3=-rhob(i)/rhobp(i,0)**2 * rhobp(i,3)**2*t(i,3)

         drhobidrix=drhobidrho0i*drhoi0drix &
              +drhobidrhob0i*drhobi0drix+drhobidrhob1i*drhobi1drix &
              +drhobidrhob2i*drhobi2drix+drhobidrhob3i*drhobi3drix &
              +drhobidti1*dti1drix+drhobidti2*dti2drix+drhobidti3*dti3drix
         drhobidriy=drhobidrho0i*drhoi0driy &
              +drhobidrhob0i*drhobi0driy+drhobidrhob1i*drhobi1driy &
              +drhobidrhob2i*drhobi2driy+drhobidrhob3i*drhobi3driy &
              +drhobidti1*dti1driy+drhobidti2*dti2driy+drhobidti3*dti3driy
         drhobidriz=drhobidrho0i*drhoi0driz &
              +drhobidrhob0i*drhobi0driz+drhobidrhob1i*drhobi1driz &
              +drhobidrhob2i*drhobi2driz+drhobidrhob3i*drhobi3driz &
              +drhobidti1*dti1driz+drhobidti2*dti2driz+drhobidti3*dti3driz

         drhob_x(i,i)=drhobidrix; drhob_y(i,i)=drhobidriy; drhob_z(i,i)=drhobidriz

! stress
         do ia=1,3; do ib=1,3
            drhob_e(i,ia,ib)=drhobidrho0i*drhoi0de(ia,ib) &
              +drhobidrhob0i*drhobi0de(ia,ib)+drhobidrhob1i*drhobi1de(ia,ib) &
              +drhobidrhob2i*drhobi2de(ia,ib)+drhobidrhob3i*drhobi3de(ia,ib) &
              +drhobidti1*dtide(1,ia,ib)+drhobidti2*dtide(2,ia,ib)+drhobidti3*dtide(3,ia,ib)
         enddo; enddo

!         if (i==1) then
!!            debug_val(1)=rhos(i)
!!            debug_val(2)=drhoi0de(1,1)
!!            debug_val(1)=t(i,2)
!!            debug_val(2)=dtide(2,1,1)
!            debug_val(1)=rhob(i)
!            debug_val(2)=drhob_e(i,1,1)
!         endif

!      end do iloop2

!      debug_val(1)=rhos(1)
!      debug_val(1)=rhob(5)
!      debug_val(1)=rhob(1)
!      debug_val(1)=rhobp(5,2)
!      debug_val(2)=drhob_x(1,1)
!      debug_val(1)=rhob(5)
!      debug_val(2)=drhob_x(5,1)

!3rd set of loops
!      kns=0
         kns=0; if (i>1) then; do icnt=1,i-1; kns=kns+knum(icnt); enddo; endif

!      iloop3: do i=1,n
!         kn=knum(i)
!         if (kn.eq.0) cycle
         kns0=kns
         rix=rx(i); riy=ry(i); riz=rz(i)
         drbidri_x = drhob_x(i,i); drbidri_y = drhob_y(i,i); drbidri_z = drhob_z(i,i)
         f1_x = -dff(i,rhob(i))*drbidri_x
         f1_y = -dff(i,rhob(i))*drbidri_y
         f1_z = -dff(i,rhob(i))*drbidri_z

         fx(i)=fx(i)+f1_x; fy(i)=fy(i)+f1_y; fz(i)=fz(i)+f1_z

         enpoteach(i)=enpoteach(i)+ff(i,rhob(i))

         jloop3: do jk=1,kn
            kns=kns+1
            j=idpt(kns,1)
            ix=idpt(kns,2); iy=idpt(kns,3); iz=idpt(kns,4)
            xx=rx(i)-rx(j); yy=ry(i)-ry(j); zz=rz(i)-rz(j)
            call calc_replica(xx,yy,zz,rc,ix,iy,iz,istat)
            rjx=rx(i)-xx; rjy=ry(i)-yy; rjz=rz(i)-zz
            if (istat == 1) then
               r2=xx**2+yy**2+zz**2
               if (r2.lt.rc2*fac) then
                  rij=sqrt(r2)

                  sij = ss(i,j); sji = sij
!                  dsjidri_x = ds_x(j,i,i); dsjidri_y = ds_y(j,i,i); dsjidri_z = ds_z(j,i,i)
                  dsjidri_x = ds_x(j,i); dsjidri_y = ds_y(j,i); dsjidri_z = ds_z(j,i)
                  dsijdri_x = dsjidri_x; dsijdri_y = dsjidri_y; dsijdri_z = dsjidri_z
                  drijdri_x = xx/rij; drijdri_y = yy/rij; drijdri_z = zz/rij
                  drjidri_x = xx/rij; drjidri_y = yy/rij; drjidri_z = zz/rij

                  drbjdri_x = drhob_x(j,i); drbjdri_y = drhob_y(j,i); drbjdri_z = drhob_z(j,i)

                  f2_x = -dff(j,rhob(j))*drbjdri_x
                  f2_y = -dff(j,rhob(j))*drbjdri_y
                  f2_z = -dff(j,rhob(j))*drbjdri_z
                  
                  fx(i)=fx(i)+f2_x; fy(i)=fy(i)+f2_y; fz(i)=fz(i)+f2_z

!                  f31_x = -(dee(i,j,rij)*sji*drijdri_x+ee(i,j,rij)*dsjidri_x)/zz0
!                  f31_y = -(dee(i,j,rij)*sji*drijdri_y+ee(i,j,rij)*dsjidri_y)/zz0
!                  f31_z = -(dee(i,j,rij)*sji*drijdri_z+ee(i,j,rij)*dsjidri_z)/zz0
!                  f32_x = -(dee(j,i,rij)*sij*drjidri_x+ee(j,i,rij)*dsijdri_x)/zz0
!                  f32_y = -(dee(j,i,rij)*sij*drjidri_y+ee(j,i,rij)*dsijdri_y)/zz0
!                  f32_z = -(dee(j,i,rij)*sij*drjidri_z+ee(j,i,rij)*dsijdri_z)/zz0
                  f31_x = -(dee(i,j,rij)*drijdri_x+ee(i,j,rij)*dsjidri_x*0)/zz0
                  f31_y = -(dee(i,j,rij)*drijdri_y+ee(i,j,rij)*dsjidri_y*0)/zz0
                  f31_z = -(dee(i,j,rij)*drijdri_z+ee(i,j,rij)*dsjidri_z*0)/zz0
                  f32_x = -(dee(j,i,rij)*drjidri_x+ee(j,i,rij)*dsijdri_x*0)/zz0
                  f32_y = -(dee(j,i,rij)*drjidri_y+ee(j,i,rij)*dsijdri_y*0)/zz0
                  f32_z = -(dee(j,i,rij)*drjidri_z+ee(j,i,rij)*dsijdri_z*0)/zz0

                  ispc=tsp(asp(i)); jspc=tsp(asp(j))
                  iii=int(dble(mesh)*(rij/ang-refxmin)/(refxmax-refxmin))
                  if (ispc==1) then
                     rhobrefi=rhobref_Si(iii); drhobrefi=drhobref_Si(iii)/ang
                  else
                     rhobrefi=rhobref_C(iii);  drhobrefi=drhobref_C(iii)/ang
                  endif
                  if (jspc==1) then
                     rhobrefj=rhobref_Si(iii); drhobrefj=drhobref_Si(iii)/ang
                  else
                     rhobrefj=rhobref_C(iii);  drhobrefj=drhobref_C(iii)/ang
                  endif
!                  f41_x = (dff(i,rhobrefj)*sji*drhobrefj*drijdri_x+ff(i,rhobrefj)*dsjidri_x)/(zz0*2.0)
!                  f41_y = (dff(i,rhobrefj)*sji*drhobrefj*drijdri_y+ff(i,rhobrefj)*dsjidri_y)/(zz0*2.0)
!                  f41_z = (dff(i,rhobrefj)*sji*drhobrefj*drijdri_z+ff(i,rhobrefj)*dsjidri_z)/(zz0*2.0)
!                  f42_x = (dff(j,rhobrefi)*sij*drhobrefi*drjidri_x+ff(j,rhobrefi)*dsijdri_x)/(zz0*2.0)
!                  f42_y = (dff(j,rhobrefi)*sij*drhobrefi*drjidri_y+ff(j,rhobrefi)*dsijdri_y)/(zz0*2.0)
!                  f42_z = (dff(j,rhobrefi)*sij*drhobrefi*drjidri_z+ff(j,rhobrefi)*dsijdri_z)/(zz0*2.0)
                  f41_x = (dff(i,rhobrefj)*drhobrefj*drijdri_x+ff(i,rhobrefj)*dsjidri_x*0)/(zz0*2.0)
                  f41_y = (dff(i,rhobrefj)*drhobrefj*drijdri_y+ff(i,rhobrefj)*dsjidri_y*0)/(zz0*2.0)
                  f41_z = (dff(i,rhobrefj)*drhobrefj*drijdri_z+ff(i,rhobrefj)*dsjidri_z*0)/(zz0*2.0)
                  f42_x = (dff(j,rhobrefi)*drhobrefi*drjidri_x+ff(j,rhobrefi)*dsijdri_x*0)/(zz0*2.0)
                  f42_y = (dff(j,rhobrefi)*drhobrefi*drjidri_y+ff(j,rhobrefi)*dsijdri_y*0)/(zz0*2.0)
                  f42_z = (dff(j,rhobrefi)*drhobrefi*drjidri_z+ff(j,rhobrefi)*dsijdri_z*0)/(zz0*2.0)

                  f51_x = f41_x; f51_y = f41_y; f51_z = f41_z
                  f52_x = f42_x; f52_y = f42_y; f52_z = f42_z

                  fx(i)=fx(i)+f31_x+f32_x+f41_x+f42_x+f51_x+f52_x
                  fy(i)=fy(i)+f31_y+f32_y+f41_y+f42_y+f51_y+f52_y
                  fz(i)=fz(i)+f31_z+f32_z+f41_z+f42_z+f51_z+f52_z

                  knsj=0; if (j>0) then; do jcnt=1,j-1; knsj=knsj+knum(jcnt); enddo; endif
                  ixj=ix;iyj=iy;izj=iz
                  knsk=kns0
                  kloop3: do kcount=1,kn   ! <to calculate S_ij>
                     knsk=knsk+1
                     k=idpt(knsk,1)
                     ixk=idpt(knsk,2); iyk=idpt(knsk,3); izk=idpt(knsk,4)
                     xx=rx(i)-rx(k); yy=ry(i)-ry(k); zz=rz(i)-rz(k)
                     call calc_replica(xx,yy,zz,rc,ixk,iyk,izk,istat)
                     rkx=rx(i)-xx; rky=ry(i)-yy; rkz=rz(i)-zz
                     rik2=(rkx-rix)**2+(rky-riy)**2+(rkz-riz)**2; if (rik2.le.eps) cycle
                     rjk2=(rkx-rjx)**2+(rky-rjy)**2+(rkz-rjz)**2; if (rjk2.le.eps) cycle
                     if (rjk2.ge.rc2*fac) cycle !; if (rik2.ge.rc2*fac) cycle; if (rji2.ge.rc2*fac) cycle
                     rjk=sqrt(rjk2); rkj=rjk

                     skj = ss(k,j); sjk = skj
!                     dskjdri_x = ds_x(k,j,i); dskjdri_y = ds_y(k,j,i); dskjdri_z = ds_z(k,j,i)
                     dskjdri_x = ds_x(k,j); dskjdri_y = ds_y(k,j); dskjdri_z = ds_z(k,j)
                     drbkdri_x = drhob_x(k,i); drbkdri_y = drhob_y(k,i); drbkdri_z = drhob_z(k,i)

                     if (rik2<rc2) then
                        f33_x = -(ee(j,k,rjk)*dskjdri_x)/zz0
                        f33_y = -(ee(j,k,rjk)*dskjdri_y)/zz0
                        f33_z = -(ee(j,k,rjk)*dskjdri_z)/zz0
                     else  ! (This may be useless because dskjdri is zero when rik>rc)
                        f33_x = -(ee(j,k,rjk)*dskjdri_x)/zz0 -(ee(k,j,rjk)*dskjdri_x)/zz0
                        f33_y = -(ee(j,k,rjk)*dskjdri_y)/zz0 -(ee(k,j,rjk)*dskjdri_y)/zz0
                        f33_z = -(ee(j,k,rjk)*dskjdri_z)/zz0 -(ee(k,j,rjk)*dskjdri_z)/zz0
                     endif
!                     f33_x=0.0; f33_y=0.0; f33_z=0.0

                     kspc=tsp(asp(k))
                     iii=int(dble(mesh)*(rjk/ang-refxmin)/(refxmax-refxmin))
                     if ((rjk/ang<refxmin).or.(rjk/ang>refxmax)) then
                        print *,'MEAM error: Rjk=',rjk/ang,' out of range of reference data'
                        stop
                     endif
                     if (kspc==1) then
                        rhobrefk=rhobref_Si(iii); drhobrefk=drhobref_Si(iii)/ang
                     else
                        rhobrefk=rhobref_C(iii);  drhobrefk=drhobref_C(iii)/ang
                     endif
                     if (rik2<rc2) then
!                        f43_x = (dff(j,rhobrefk)*skj*drbkdri_x+ff(j,rhobrefk)*dskjdri_x)/(zz0*2.0)
!                        f43_y = (dff(j,rhobrefk)*skj*drbkdri_y+ff(j,rhobrefk)*dskjdri_y)/(zz0*2.0)
!                        f43_z = (dff(j,rhobrefk)*skj*drbkdri_z+ff(j,rhobrefk)*dskjdri_z)/(zz0*2.0)
!                        f53_x = (dff(k,rhobrefj)*skj*drbjdri_x+ff(k,rhobrefj)*dskjdri_x)/(zz0*2.0)
!                        f53_y = (dff(k,rhobrefj)*skj*drbjdri_y+ff(k,rhobrefj)*dskjdri_y)/(zz0*2.0)
!                        f53_z = (dff(k,rhobrefj)*skj*drbjdri_z+ff(k,rhobrefj)*dskjdri_z)/(zz0*2.0)
                        f43_x = (ff(j,rhobrefk)*dskjdri_x)/(zz0*2.0)
                        f43_y = (ff(j,rhobrefk)*dskjdri_y)/(zz0*2.0)
                        f43_z = (ff(j,rhobrefk)*dskjdri_z)/(zz0*2.0)
                        f53_x = (ff(k,rhobrefj)*dskjdri_x)/(zz0*2.0)
                        f53_y = (ff(k,rhobrefj)*dskjdri_y)/(zz0*2.0)
                        f53_z = (ff(k,rhobrefj)*dskjdri_z)/(zz0*2.0)
                     else  ! if k is out of the cut-off range, f53 must be double-counted here.
                        f43_x = 0.0
                        f43_y = 0.0
                        f43_z = 0.0
!                        f53_x = (dff(k,rhobrefj)*skj*drbjdri_x+ff(k,rhobrefj)*dskjdri_x)/(zz0*2.0)*2.0
!                        f53_y = (dff(k,rhobrefj)*skj*drbjdri_y+ff(k,rhobrefj)*dskjdri_y)/(zz0*2.0)*2.0
!                        f53_z = (dff(k,rhobrefj)*skj*drbjdri_z+ff(k,rhobrefj)*dskjdri_z)/(zz0*2.0)*2.0
                        f53_x = (ff(k,rhobrefj)*dskjdri_x)/(zz0*2.0)*2.0
                        f53_y = (ff(k,rhobrefj)*dskjdri_y)/(zz0*2.0)*2.0
                        f53_z = (ff(k,rhobrefj)*dskjdri_z)/(zz0*2.0)*2.0
                     endif

!                     fx(i)=fx(i)+f33_x+f43_x+f53_x
!                     fy(i)=fy(i)+f33_y+f43_y+f53_y
!                     fz(i)=fz(i)+f33_z+f43_z+f53_z

                  enddo kloop3

!                  enpoteach(i)=enpoteach(i)+(ee(i,j,rij)*2.0-ff(i,rhobrefj)-ff(j,rhobrefi))*ss(i,j)/zz0/2.0
                  enpoteach(i)=enpoteach(i)+(ee(i,j,rij)*2.0-ff(i,rhobrefj)-ff(j,rhobrefi))/zz0/2.0
!                  enpoteach(i)=enpoteach(i)+(ee(i,j,rij)*2.0)/zz0/2.0

               endif
            endif
         end do jloop3
!      end do iloop3
      end do iloop2

!Final set of loops for stress calculation
      kns=0
      iloop0: do i=1,n
         kn=knum(i)
         if (kn.eq.0) cycle
         kns0=kns
         rix=rx(i); riy=ry(i); riz=rz(i)

         do ia=1,3; do ib=1,3 !stress
            satom(i,ia,ib)=satom(i,ia,ib)+dff(i,rhob(i))*drhob_e(i,ia,ib)
         enddo; enddo

         jloop0: do jk=1,kn
            kns=kns+1
            j=idpt(kns,1)
            ix=idpt(kns,2); iy=idpt(kns,3); iz=idpt(kns,4)
            xx=rx(i)-rx(j); yy=ry(i)-ry(j); zz=rz(i)-rz(j)
            call calc_replica(xx,yy,zz,rc,ix,iy,iz,istat)
            rjx=rx(i)-xx; rjy=ry(i)-yy; rjz=rz(i)-zz
            if (istat == 0) cycle
            r2=xx**2+yy**2+zz**2
            if (r2.ge.rc2*fac) cycle
            rij=sqrt(r2)
            sji=ss(j,i)
            
            ispc=tsp(asp(i)); jspc=tsp(asp(j))
            iii=int(dble(mesh)*(rij/ang-refxmin)/(refxmax-refxmin))
            if (ispc==1) then
               rhobrefi=rhobref_Si(iii); drhobrefi=drhobref_Si(iii)/ang
            else
               rhobrefi=rhobref_C(iii);  drhobrefi=drhobref_C(iii)/ang
            endif
            if (jspc==1) then
               rhobrefj=rhobref_Si(iii); drhobrefj=drhobref_Si(iii)/ang
            else
               rhobrefj=rhobref_C(iii);  drhobrefj=drhobref_C(iii)/ang
            endif
            do ia=1,3; do ib=1,3 !stress
               satom(i,ia,ib)=satom(i,ia,ib) &
                    +(dee(i,j,rij)*drde(rjx-rix,rjy-riy,rjz-riz,rij,ia,ib)*sji+ee(i,j,rij)*ds_e(j,i,ia,ib))/zz0 &
                    -(dff(i,rhobrefj)*drhob_e(j,ia,ib)*sji+ff(i,rhobrefj)*ds_e(j,i,ia,ib))/zz0/2.0 &
                    -(dff(j,rhobrefi)*drhob_e(i,ia,ib)*sji+ff(j,rhobrefi)*ds_e(j,i,ia,ib))/zz0/2.0
            enddo; enddo
            
!            knsk=kns0
!            kloop0: do kcount=1,kn
!               knsk=knsk+1
!               k=idpt(knsk,1)
!               ixk=idpt(knsk,2); iyk=idpt(knsk,3); izk=idpt(knsk,4)
!               xx=rx(i)-rx(k); yy=ry(i)-ry(k); zz=rz(i)-rz(k)
!               call calc_replica(xx,yy,zz,rc,ixk,iyk,izk,istat)
!               rkx=rx(i)-xx; rky=ry(i)-yy; rkz=rz(i)-zz
!               rik2=(rkx-rix)**2+(rky-riy)**2+(rkz-riz)**2; if (rik2.le.eps) cycle
!               rjk2=(rkx-rjx)**2+(rky-rjy)**2+(rkz-rjz)**2; if (rjk2.le.eps) cycle
!               if (rik2.ge.rc2*fac) cycle
!               rjk=sqrt(rjk2); rkj=rjk
!            enddo kloop0
            
         end do jloop0
!         print *,i,satom(i,1,1)
      end do iloop0

!---Etot---
      enpot=0.0d0
      do i=1,n
         enpot=enpot+enpoteach(i)
      enddo
!      debug_val(1)=-enpot
!      debug_val(2)=fy(1)

!!     Atomic stress
!      do i=1,n
!         satom(i,2,1)=satom(i,1,2)
!         satom(i,3,1)=satom(i,1,3)
!         satom(i,3,2)=satom(i,2,3)
!      enddo
      dmat=0.0d0
! The sign of dmat is opposite to stress tensor's (dmat is the "virial" tensor)
      do i=1,n
         dmat(1:3,1:3)=dmat(1:3,1:3)-satom(i,1:3,1:3)
      enddo
      virx=dmat(1,1); viry=dmat(2,2); virz=dmat(3,3)

      satom=satom*dble(n)/v

!      stop
    end subroutine meam_HuangSiC

    subroutine meam_HuangSiC_setup
      use meam_HuangSiC_function
      use variables
      use constants
      implicit double precision(a-h,o-z)
      character (1) :: ichar
      integer :: meshref

!==Si==
! open file test, count line number, allocate data
      open (10,file='pot/meam_si_ref.d',status='old',err=999)
      meshref=0
10    read (10,*,end=30) ichar !trial read
      if (ichar(:1)=='#') then
         go to 10               !skip a line if it is a comment line
      else
         backspace 10           !read again if it is not a comment line
      endif
20    read (10,*,end=30,err=10) a1,a2
      meshref=meshref+1
      go to 20
30    close (10)
      allocate (rhobrefx(meshref),rhobrefy(meshref),rhobrefc(meshref,3))

! read data from file
      open (10,file='pot/meam_si_ref.d',status='old')
      meshref=0
 15   read (10,*,end=35) ichar !trial read
      if (ichar(:1)=='#') then
         go to 15               !skip a line if it is a comment line
      else
         backspace 10           !read again if it is not a comment line
      endif
 25   read (10,*,end=35,err=15) a1,a2
      meshref=meshref+1
      if (meshref==1) refxmin=a1
      rhobrefx(meshref)=a1
      rhobrefy(meshref)=a2
      go to 25
 35   close (10)
      refxmax=a1
! finer mesh set
      allocate(rhobrefxn(mesh),spd2(mesh))
      x=refxmin
      d=(refxmax-refxmin)/dble(mesh-1)
      do i=1,mesh
         rhobrefxn(i)=x
         x=x+d
      enddo
! function set
      iopt(1)=3; iopt(2)=3
      call SPLC(rhobrefx, meshref, rhobrefy, DF, IOPT, rhobrefc, meshref, IER)
      call SPLF(rhobrefx,meshref,rhobrefy,rhobrefc,meshref,rhobrefxn,mesh,rhobref_Si, IER)
      call SPLD(rhobrefx,meshref,rhobrefc,meshref,rhobrefxn,mesh,drhobref_Si,spd2, IER)
      deallocate (rhobrefxn,spd2)
      deallocate (rhobrefx,rhobrefy,rhobrefc)
!==C==
! open file test, count line number, allocate data
      open (10,file='pot/meam_c_ref.d',status='old',err=999)
      meshref=0
110   read (10,*,end=130) ichar !trial read
      if (ichar(:1)=='#') then
         go to 110              !skip a line if it is a comment line
      else
         backspace 10           !read again if it is not a comment line
      endif
120   read (10,*,end=130,err=110) a1,a2
      meshref=meshref+1
      go to 120
130   close (10)
      allocate (rhobrefx(meshref),rhobrefy(meshref),rhobrefc(meshref,3))
! read data from file
      open (10,file='pot/meam_c_ref.d',status='old')
      meshref=0
115   read (10,*,end=135) ichar !trial read
      if (ichar(:1)=='#') then
         go to 115              !skip a line if it is a comment line
      else
         backspace 10           !read again if it is not a comment line
      endif
125   read (10,*,end=135,err=115) a1,a2
      meshref=meshref+1
      if (meshref==1) refxmin=a1
      rhobrefx(meshref)=a1
      rhobrefy(meshref)=a2
      go to 125
135   close (10)
      refxmax=a1
! finer mesh set
      allocate(rhobrefxn(mesh),spd2(mesh))
      x=refxmin
      d=(refxmax-refxmin)/dble(mesh-1)
      do i=1,mesh
         rhobrefxn(i)=x
         x=x+d
      enddo
! function set
      iopt(1)=3; iopt(2)=3
      call SPLC(rhobrefx, meshref, rhobrefy, DF, IOPT, rhobrefc, meshref, IER)
      call SPLF(rhobrefx,meshref,rhobrefy,rhobrefc,meshref,rhobrefxn,mesh,rhobref_C, IER)
      call SPLD(rhobrefx,meshref,rhobrefc,meshref,rhobrefxn,mesh,drhobref_C,spd2, IER)
      deallocate (rhobrefxn,spd2)
      deallocate (rhobrefx,rhobrefy,rhobrefc)

      meam_HuangSiC_setup_done = .true.
      print *,'MEAM setup done. meam_si_ref.d and meam_c_ref.d have been read.'
      return
999   print *,'MEAM setup error: reference data for Si/C is not found.'
      stop
    end subroutine meam_HuangSiC_setup
