      subroutine readconfig
      use variables
      use constants
      implicit double precision (a-h,o-z)
      double precision :: xx,yy,zz
      character (1) :: ifx,ify,ifz
      character (3) :: fmt
      character (2) :: species
      character (15) :: arg(8)
      integer :: narg, n0
      
      if (multicellx<=0) multicellx = 1
      if (multicelly<=0) multicelly = 1
      if (multicellz<=0) multicellz = 1

      open (30,FILE='CONFIG',status='old',action='read',iostat=ierr)
      if (ierr/=0) then
         print *,'CANNOT FIND <CONFIG> FILE'
         return
      endif
      if (ierr==0) then
         print *,'READ CONFIG'
         read (30,*) species
!         read (30,*) potential_func
         call getargs(30,arg,narg)
         potential_func=TRIM(arg(1))
         if (narg/=1) then
            potential_arg=TRIM(arg(2))
         else
            potential_arg=''
         endif
         read (30,*) n0
         n=n0*multicellx*multicelly*multicellz
         allocate (asp(n),wm(n))
         multisystem=0
         if ((species == 'Mu').or.(species == 'mu')) multisystem=1
         if (multisystem==0) then
            asp=species
         endif
         read (30,*) scl_lat
         deallocate (rx,ry,rz,vx,vy,vz,ax,ay,az,bx,by,bz,cx,cy,cz)
         deallocate (fx,fy,fz,qx,qy,qz)
         deallocate (enpoteach,enkineach)
         allocate (enpoteach(n),enkineach(n))
         allocate (rx(n),ry(n),rz(n)) ! Positions
         if (diffusion) allocate (rxd(n),ryd(n),rzd(n)) ! Positions
!         if (diffusion) allocate (rxp(n),ryp(n),rzp(n)) ! Positions
         allocate (vx(n),vy(n),vz(n)) ! Velocities
         allocate (fx(n),fy(n),fz(n)) ! Forces
         allocate (qx(n),qy(n),qz(n)) ! Normalized positions
         allocate (ax(n),ay(n),az(n)) ! Accerelations (Second derivatives)
         allocate (bx(n),by(n),bz(n)) ! Third derivatives
         allocate (cx(n),cy(n),cz(n)) ! Fourth derivatives
         allocate (mfx(n),mfy(n),mfz(n)) ! Motion control
         allocate (qpx(n),qpy(n),qpz(n)) ! Positions for memory
         allocate (satom(n,3,3)) ! Atomic stress
         px=0.0d0; py=0.0d0; pz=0.0d0
         fx=0.0d0; fy=0.0d0; fz=0.0d0
         vx=0.0d0; vy=0.0d0; vz=0.0d0
         ax=0.0d0; ay=0.0d0; az=0.0d0
         bx=0.0d0; by=0.0d0; bz=0.0d0
         cx=0.0d0; cy=0.0d0; cz=0.0d0
         satom=0.0d0
         do j=1,3
            read (30,*) xx,yy,zz
            hmat(1,j)=xx*ang; hmat(2,j)=yy*ang; hmat(3,j)=zz*ang
         enddo
         hmat=hmat*scl_lat
         read (30,*) fmt
         if (multisystem==0) then
            do i=1,n0
               call getargs(30,arg,narg)
               backspace 30
               if (narg==3) then
                  read (30,*) xx,yy,zz
                  rx(i)=xx; ry(i)=yy; rz(i)=zz
                  mfx(i)='T';mfy(i)='T';mfz(i)='T'
               else if (narg==4) then
                  read (30,*) xx,yy,zz,species
                  rx(i)=xx; ry(i)=yy; rz(i)=zz; asp(i)=species
                  mfx(i)='T';mfy(i)='T';mfz(i)='T'
               else if (narg==6) then
                  read (30,*) xx,yy,zz,ifx,ify,ifz
                  rx(i)=xx; ry(i)=yy; rz(i)=zz
                  mfx(i)=ifx;mfy(i)=ify;mfz(i)=ifz
                  if ((ifx/='T').and.(ifx/='F')) then
                     mfx(i)='T';print *,'x motion control is not correct for i=',i;iwarn=iwarn+1
                  endif
                  if ((ify/='T').and.(ify/='F')) then
                     mfy(i)='T';print *,'y motion control is not correct for i=',i;iwarn=iwarn+1
                  endif
                  if ((ifz/='T').and.(ifz/='F')) then
                     mfz(i)='T';print *,'z motion control is not correct for i=',i;iwarn=iwarn+1
                  endif
               else if (narg==7) then
                  read (30,*) xx,yy,zz,species,ifx,ify,ifz
                  rx(i)=xx; ry(i)=yy; rz(i)=zz; asp(i)=species
                  mfx(i)=ifx;mfy(i)=ify;mfz(i)=ifz
                  if ((ifx/='T').and.(ifx/='F')) then
                     mfx(i)='T';print *,'x motion control is not correct for i=',i;iwarn=iwarn+1
                  endif
                  if ((ify/='T').and.(ify/='F')) then
                     mfy(i)='T';print *,'y motion control is not correct for i=',i;iwarn=iwarn+1
                  endif
                  if ((ifz/='T').and.(ifz/='F')) then
                     mfz(i)='T';print *,'z motion control is not correct for i=',i;iwarn=iwarn+1
                  endif
               else
                  print *,'Read CONFIG error: line cannot be read correctly'
                  print *,'i=',i,' line has',narg,' parameters'
                  stop
               endif
            enddo
         else
            do i=1,n0
               call getargs(30,arg,narg)
               backspace 30
               if (narg==4) then
                  read (30,*) xx,yy,zz,species
                  rx(i)=xx; ry(i)=yy; rz(i)=zz; asp(i)=species
                  mfx(i)='T';mfy(i)='T';mfz(i)='T'
               else if (narg==7) then
                  read (30,*) xx,yy,zz,species,ifx,ify,ifz
                  rx(i)=xx; ry(i)=yy; rz(i)=zz; asp(i)=species
                  mfx(i)=ifx;mfy(i)=ify;mfz(i)=ifz
                  if ((ifx/='T').and.(ifx/='F')) then
                     mfx(i)='T';print *,'x motion control is not correct for i=',i;iwarn=iwarn+1
                  endif
                  if ((ify/='T').and.(ify/='F')) then
                     mfy(i)='T';print *,'y motion control is not correct for i=',i;iwarn=iwarn+1
                  endif
                  if ((ifz/='T').and.(ifz/='F')) then
                     mfz(i)='T';print *,'z motion control is not correct for i=',i;iwarn=iwarn+1
                  endif
               else
                  print *,'Read CONFIG error: line cannot be read correctly'
                  print *,'i=',i,' line has',narg,' parameters'
                  stop
               endif
            enddo
         endif
         if (fmt=='abs') then   ! cartesian coordinate
            rx=rx*ang; ry=ry*ang; rz=rz*ang
         else                   ! fractional coordinate
            do i=1,n0
               xx=rx(i); yy=ry(i); zz=rz(i)
               rx(i)=hmat(1,1)*xx+hmat(1,2)*yy+hmat(1,3)*zz
               ry(i)=hmat(2,1)*xx+hmat(2,2)*yy+hmat(2,3)*zz
               rz(i)=hmat(3,1)*xx+hmat(3,2)*yy+hmat(3,3)*zz
            enddo
         endif
         close (30)
         multisystem=0
         do i=1+1,n; if (asp(i)/=asp(1)) multisystem=1;enddo
         if (multisystem==1) then
            print *,'Multisystem',' Potential:',potential_func,' ',potential_arg
         else
            asp=species
            print *,'Atom:',species,' Potential:',potential_func,' ',potential_arg
         endif

! Multiply cell size
         if (multicellx>1) then
            do iix=2,multicellx
               do i=1,n0
                  ii=(iix-1)*n0+i
                  rx(ii)=rx(i)+hmat(1,1)*dble(iix-1)
                  ry(ii)=ry(i)+hmat(2,1)*dble(iix-1)
                  rz(ii)=rz(i)+hmat(3,1)*dble(iix-1)
                  asp(ii)=asp(i);mfx(ii)=mfx(i);mfy(ii)=mfy(i);mfz(ii)=mfz(i)
               enddo
            enddo
            n0=n0*multicellx
            hmat(1,1)=hmat(1,1)*dble(multicellx)
            hmat(2,1)=hmat(2,1)*dble(multicellx)
            hmat(3,1)=hmat(3,1)*dble(multicellx)
         endif
         if (multicelly>1) then
            do iiy=2,multicelly
               do i=1,n0
                  ii=(iiy-1)*n0+i
                  rx(ii)=rx(i)+hmat(1,2)*dble(iiy-1)
                  ry(ii)=ry(i)+hmat(2,2)*dble(iiy-1)
                  rz(ii)=rz(i)+hmat(3,2)*dble(iiy-1)
                  asp(ii)=asp(i);mfx(ii)=mfx(i);mfy(ii)=mfy(i);mfz(ii)=mfz(i)
               enddo
            enddo
            n0=n0*multicelly
            hmat(1,2)=hmat(1,2)*dble(multicelly)
            hmat(2,2)=hmat(2,2)*dble(multicelly)
            hmat(3,2)=hmat(3,2)*dble(multicelly)
         endif
         if (multicellz>1) then
            do iiz=2,multicellz
               do i=1,n0
                  ii=(iiz-1)*n0+i
                  rx(ii)=rx(i)+hmat(1,3)*dble(iiz-1)
                  ry(ii)=ry(i)+hmat(2,3)*dble(iiz-1)
                  rz(ii)=rz(i)+hmat(3,3)*dble(iiz-1)
                  asp(ii)=asp(i);mfx(ii)=mfx(i);mfy(ii)=mfy(i);mfz(ii)=mfz(i)
               enddo
            enddo
            n0=n0*multicellx
            hmat(1,3)=hmat(1,3)*dble(multicellz)
            hmat(2,3)=hmat(2,3)*dble(multicellz)
            hmat(3,3)=hmat(3,3)*dble(multicellz)
         endif

         call inverse(hmat,hinmat)
         call calcvolume
         call mkq
      endif

      end

      subroutine getargs(filenum,arg,narg)
      integer :: filenum, narg
      character (100) :: lineread
      character (15) :: arg(8)

      read (filenum,"(a100)") lineread
      iarg=1
      do ii=1,100
         if (lineread(ii:ii)==' ') cycle
         exit
      enddo
      i=ii
      io=ii
      do while (i<=100) 
         if (lineread(i:i)==' ') then
            narg=iarg
            arg(iarg)=lineread(io:i-1)
            iarg=iarg+1
            do ii=i+1,100
               if (lineread(ii:ii)==' ') cycle
               exit
            enddo
            i=ii-1
            io=ii
         endif
         if (iarg>8) exit
         i=i+1
      enddo
!      do i=1,narg
!         print *,'arg',i,'=',TRIM(arg(i))
!      enddo
      end

      subroutine read_cfge(i,filename)
        use variables
        use constants
        integer :: ifile
        character(*) :: filename
        open (i,FILE=filename)
        do ii=1,17
           read (i,*)
        enddo
        do ii=1,n
           read (i,*);read (i,*)
           read (i,*) qpx(ii),qpy(ii),qpz(ii)
        enddo
        close (i)
      end subroutine read_cfge
