      program instvis
      implicit double precision (a-h,o-z)
      double precision, allocatable :: rx(:),ry(:),rz(:),vec(:)
      double precision, allocatable :: qx(:),qy(:),qz(:)
      double precision :: hmat(3,3),hinmat(3,3)
      double precision :: xx,yy,zz,scl
      double precision :: ang=1.0d-10
      character  :: dmy
      character(12) :: filename
      character (3) :: fmt
      character (2) :: atom_species = 'Fe'
      integer :: iatom = 26

      open (30,FILE='CONFIG.END.FRA',status='old',action='read',iostat=ierr)
      if (ierr/=0) then
         stop 'CANNOT FIND <CONFIG> FILE'
      endif
      if (ierr==0) then
         read (30,*) dmy
         read (30,*) dmy
         read (30,*) n
         read (30,*) scl
         allocate (rx(n),ry(n),rz(n),vec(n))
         allocate (qx(n),qy(n),qz(n))
         do j=1,3
            read (30,*) xx,yy,zz
            hmat(1,j)=xx*ang; hmat(2,j)=yy*ang; hmat(3,j)=zz*ang
         enddo
         hmat=hmat*scl
         read (30,*) fmt
         do i=1,n
            read (30,*) xx,yy,zz
            rx(i)=xx; ry(i)=yy; rz(i)=zz
         enddo
         if (fmt=='abs') then   ! cartesian coordinate
            rx=rx*ang; ry=ry*ang; rz=rz*ang
         else                   ! fractional coordinate
            do i=1,n
               xx=rx(i); yy=ry(i); zz=rz(i)
               rx(i)=hmat(1,1)*xx+hmat(1,2)*yy+hmat(1,3)*zz
               ry(i)=hmat(2,1)*xx+hmat(2,2)*yy+hmat(2,3)*zz
               rz(i)=hmat(3,1)*xx+hmat(3,2)*yy+hmat(3,3)*zz
            enddo
         endif
         close (30)
      endif

      open (30,FILE='eigen_atom.d',status='old',action='read',iostat=ierr)
      if (ierr/=0) then
         stop 'CANNOT FIND <CONFIG> FILE'
      endif
      if (ierr==0) then
         read (30,*) evalue
         read (30,*) n
         do i=1,n
            read (30,*) xx,yy,zz
            vec(i)=sqrt(xx**2+yy**2+zz**2)
         enddo
         close (30)
      endif

      filename='atominst.cfg'
      ifile=10

      open (ifile,FILE=filename)
      write (ifile,'(a22,i10)') 'Number of particles = ',n
      write (ifile,'(a4,f10.5,a9)') 'A = ',1.0,' Angstrom'
      write (ifile,'(a10,f10.5)') 'H0(1,1) = ',hmat(1,1)/ang
      write (ifile,'(a10,f10.5)') 'H0(1,2) = ',hmat(2,1)/ang
      write (ifile,'(a10,f10.5)') 'H0(1,3) = ',hmat(3,1)/ang
      write (ifile,'(a10,f10.5)') 'H0(2,1) = ',hmat(1,2)/ang
      write (ifile,'(a10,f10.5)') 'H0(2,2) = ',hmat(2,2)/ang
      write (ifile,'(a10,f10.5)') 'H0(2,3) = ',hmat(3,2)/ang
      write (ifile,'(a10,f10.5)') 'H0(3,1) = ',hmat(1,3)/ang
      write (ifile,'(a10,f10.5)') 'H0(3,2) = ',hmat(2,3)/ang
      write (ifile,'(a10,f10.5)') 'H0(3,3) = ',hmat(3,3)/ang
      write (ifile,'(a13)') '.NO_VELOCITY.'
      write (ifile,'(a15)') 'entry_count = 4'
      write (ifile,'(a36)') 'auxiliary[0] = evector[reduced unit]'
      write (ifile,'(f10.5)') 1.0000
      write (ifile,'(a2)') atom_species

      call inverse(hmat,hinmat)
      do k=1,n
         qx(k)=hinmat(1,1)*rx(k)+hinmat(1,2)*ry(k)+hinmat(1,3)*rz(k)
         qy(k)=hinmat(2,1)*rx(k)+hinmat(2,2)*ry(k)+hinmat(2,3)*rz(k)
         qz(k)=hinmat(3,1)*rx(k)+hinmat(3,2)*ry(k)+hinmat(3,3)*rz(k)
      enddo

      do i=1,n
         write (ifile,'(4f12.8)') qx(i),qy(i),qz(i),vec(i)
      enddo

      close (ifile)

      end
      subroutine inverse(a,b)
      implicit double precision (a-h,o-z)
      double precision a(3,3),b(3,3)
      do i=1,3
         do j=1,3
            b(i,j)=a(i,j)
         enddo
      enddo
      do 100 k=1,3
         p=b(k,k)
         b(k,k)=1.0d0
         do 1 j=1,3
            b(k,j)=b(k,j)/p
 1       continue
         do 2 i=1,3
            if (i.ne.k) then
               q=b(i,k)
               b(i,k)=0.0d0
               do 3 j=1,3
                  b(i,j)=b(i,j)-q*b(k,j)
 3             continue
            endif
 2       continue
 100  continue
      return
      end
